// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Box.H>
#include "guile_fltk.h"
#include "guile_support.h"

long fltk_box_tag;

//***************************************************************************
//   
//  Fl_Box encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_box_new (SCM t, SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Box * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(t), t,  SCM_ARG1, "fltk-box-new");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-box-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-box-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG4, "fltk-box-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG5, "fltk-box-new");
  SCM_ASSERT (SCM_NIMP (label) && SCM_STRINGP (label), label,
	      SCM_ARG6, "fltk-box-new");

  widget = new Fl_Box((Fl_Boxtype)gh_scm2int(t), gh_scm2int(x), gh_scm2int(y), 
		      gh_scm2int(w), gh_scm2int(h),
		      gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_box_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Box *) SCM_CDR (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Box) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Box *widget = (Fl_Box *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Box ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-box scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_box_init ()
{
  fltk_box_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-box", fltk_box_tag);

  // add new methods
  gh_new_procedure6_0_0 ("fltk-box-new", 
		       fltk_box_new);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_box_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_box_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-box-new"));
}
