// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Browser.H>
#include "guile_fltk.h"

long fltk_browser_tag;

//***************************************************************************
//   
//  Fl_Browser encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_browser_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Browser * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-browser-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-browser-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-browser-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-browser-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-browser-new");

  widget = new Fl_Browser(gh_scm2int(x), gh_scm2int(y), 
			  gh_scm2int(w), gh_scm2int(h),
			  gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_browser_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_browser_add (SCM widget_smob, SCM a, SCM b)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-add");
  SCM_ASSERT (gh_string_p(a), a, SCM_ARG2, "fltk-browser-add");
  SCM_ASSERT (1, b, SCM_ARG3, "fltk-browser-add");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->add(gh_scm2newstr(a, NULL), (void *)b);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_clear (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-add");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->clear();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_set_column_char (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-column-char");
  SCM_ASSERT (gh_char_p(a), a, SCM_ARG2, "fltk-browser-set-column-char");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->column_char(gh_scm2char(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_column_char (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-column-char");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return gh_char2scm(widget->column_char());
}

static SCM
fltk_browser_get_column_widths (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-column-widths");

  widget = (Fl_Browser *) gh_cdr(widget_smob);
  const int *W = widget->column_widths();
  int j, i = 0;

  while (W[i] != 0) i++;

  SCM rvalue = gh_make_vector(gh_scm2int(i), gh_scm2int(0));

  for (j=0; j<i; j++)
    rvalue = gh_vector_set_x (rvalue, gh_int2scm(j), gh_int2scm(W[j]));

  return (rvalue);
}

static SCM
fltk_browser_set_column_widths (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-column-widths");
  SCM_ASSERT ((SCM_NIMP (a)), a, SCM_ARG2, "fltk-browser-get-column-widths");

  long length = gh_vector_length (a);

  widget = (Fl_Browser *) gh_cdr(widget_smob);
 
  //TODO: When do I need to delete this?
  int *W = new int[length+1];
  for (int i=0; i<length; i++)
      W[i] = gh_scm2int(gh_vector_ref(a, gh_int2scm(i)));
  W[length] = 0;

  widget->column_widths(W);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_set_data (SCM widget_smob, SCM a, SCM b)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-data");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-set-data");
  SCM_ASSERT (gh_number_p(1), b,  SCM_ARG3, "fltk-browser-set-data");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->data(gh_scm2int(a), (void *)b);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_data (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-data");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-get-data");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return (SCM)widget->data(gh_scm2int(a));
}

static SCM
fltk_browser_set_format_char (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-format-char");
  SCM_ASSERT (gh_char_p(a), a, SCM_ARG2, "fltk-browser-set-format-char");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->format_char(gh_scm2char(a));
  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_format_char (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-format-char");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return gh_char2scm(widget->format_char());
}

static SCM
fltk_browser_hide (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-add");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-hide");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->hide(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_insert (SCM widget_smob, SCM a, SCM b, SCM c)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-insert");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-insert");
  SCM_ASSERT (SCM_NIMP (b) && SCM_STRINGP (b), b,
	      SCM_ARG3, "fltk-browser-insert");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->insert(gh_scm2int(a), strdup(SCM_CHARS(b)), (void *)c);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_load (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-load");
  SCM_ASSERT (gh_string_p(a), a, SCM_ARG2, "fltk-browser-load");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  if (widget->load(SCM_CHARS(a)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_browser_move (SCM widget_smob, SCM a, SCM b)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-insert");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-move");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-browser-move");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->move(gh_scm2int(a), gh_scm2int(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_set_position (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-position");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-set-position");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->position(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_position (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-position");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return gh_int2scm(widget->position());
}

static SCM
fltk_browser_remove (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-remove");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-remove");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->remove(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_show (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-show");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-show");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->show(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_size (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-size");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return gh_int2scm(widget->size());
}

static SCM
fltk_browser_set_text (SCM widget_smob, SCM a, SCM b)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-text");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-set-text");
  SCM_ASSERT (SCM_NIMP (b) && SCM_STRINGP (b), b,
	      SCM_ARG3, "fltk-browser-set-text");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->text(gh_scm2int(a), strdup(SCM_CHARS(b)));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_text (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-text");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-text");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return scm_makfrom0str (widget->text(gh_scm2int(a)));
}

static SCM
fltk_browser_set_topline (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-topline");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-set-topline");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  widget->topline(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_get_topline (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-topline");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  return gh_int2scm(widget->topline());
}

static SCM
fltk_browser_visible (SCM widget_smob, SCM sa)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-browser-visible");
  SCM_ASSERT (gh_number_p(sa), sa,  SCM_ARG2, "fltk-browser-visible");

  widget = (Fl_Browser *) gh_cdr(widget_smob);

  if (widget->visible(gh_scm2int(sa)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Browser *) gh_cdr(widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Browser) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Browser *widget = (Fl_Browser *) gh_cdr(widget_smob);
  scm_puts ("#<widget Fl_Browser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-browser scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_browser_init ()
{
  fltk_browser_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_browser_base_subclass("fltk-browser", fltk_browser_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-browser-new", 
		       fltk_browser_new);
  gh_new_procedure3_0 ("fltk-browser-add", 
		       fltk_browser_add);
  gh_new_procedure1_0 ("fltk-browser-clear", 
		       fltk_browser_clear);
  gh_new_procedure2_0 ("fltk-browser-set-column-char", 
		       fltk_browser_set_column_char);
  gh_new_procedure1_0 ("fltk-browser-get-column-char", 
		       fltk_browser_get_column_char);
  gh_new_procedure1_0 ("fltk-browser-get-column-widths", 
		       fltk_browser_get_column_widths);
  gh_new_procedure2_0 ("fltk-browser-set-column-widths", 
		       fltk_browser_set_column_widths);
  gh_new_procedure3_0 ("fltk-browser-set-data", 
		       fltk_browser_set_data);
  gh_new_procedure2_0 ("fltk-browser-get-data", 
		       fltk_browser_get_data);
  gh_new_procedure2_0 ("fltk-browser-set-format-char", 
		       fltk_browser_set_format_char);
  gh_new_procedure1_0 ("fltk-browser-get-format-char", 
		       fltk_browser_get_format_char);
  gh_new_procedure2_0 ("fltk-browser-hide", 
		       fltk_browser_hide);
  gh_new_procedure4_0 ("fltk-browser-insert", 
		       fltk_browser_insert);
  gh_new_procedure2_0 ("fltk-browser-load", 
		       fltk_browser_load);
  gh_new_procedure3_0 ("fltk-browser-move", 
		       fltk_browser_move);
  gh_new_procedure2_0 ("fltk-browser-set-position", 
		       fltk_browser_set_position);
  gh_new_procedure1_0 ("fltk-browser-get-position", 
		       fltk_browser_get_position);
  gh_new_procedure2_0 ("fltk-browser-remove", 
		       fltk_browser_remove);
  gh_new_procedure2_0 ("fltk-browser-show", 
		       fltk_browser_show);
  gh_new_procedure1_0 ("fltk-browser-get-size", 
		       fltk_browser_get_size);
  gh_new_procedure3_0 ("fltk-browser-set_text", 
		       fltk_browser_set_text);
  gh_new_procedure2_0 ("fltk-browser-get-text", 
		       fltk_browser_get_text);
  gh_new_procedure1_0 ("fltk-browser-get-topline", 
		       fltk_browser_get_topline);
  gh_new_procedure2_0 ("fltk-browser-set-topline", 
		       fltk_browser_set_topline);
  gh_new_procedure2_0 ("fltk-browser-visible", 
		       fltk_browser_visible);

  gh_define("FL-NORMAL-BROWSER", gh_long2scm(FL_NORMAL_BROWSER));
  gh_define("FL-SELECT-BROWSER", gh_long2scm(FL_SELECT_BROWSER));
  gh_define("FL-HOLD-BROWSER", gh_long2scm(FL_HOLD_BROWSER));
  gh_define("FL-MULTI-BROWSER", gh_long2scm(FL_MULTI_BROWSER));
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_browser_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_browser_tag);
  fltk_browser_base_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-browser-new"));
  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-browser-new"));
  sprintf (t, "%s-add", s);
  gh_define (t, scm_symbol_value0("fltk-browser-add"));
  sprintf (t, "%s-clear", s);
  gh_define (t, scm_symbol_value0("fltk-browser-clear"));
  sprintf (t, "%s-set-column-char", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set-column-char"));
  sprintf (t, "%s-get-column-char", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-column-char"));
  sprintf (t, "%s-get-column-widths", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-column-widths"));
  sprintf (t, "%s-set-column-widths", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set-column-widths"));
  sprintf (t, "%s-set-data", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set-data"));
  sprintf (t, "%s-get-data", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-data"));
  sprintf (t, "%s-set-format-char", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set-format-char"));
  sprintf (t, "%s-get-format-char", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-format-char"));
  sprintf (t, "%s-hide", s);
  gh_define (t, scm_symbol_value0("fltk-browser-hide"));
  sprintf (t, "%s-insert", s);
  gh_define (t, scm_symbol_value0("fltk-browser-insert"));
  sprintf (t, "%s-load", s);
  gh_define (t, scm_symbol_value0("fltk-browser-load"));
  sprintf (t, "%s-move", s);
  gh_define (t, scm_symbol_value0("fltk-browser-move"));
  sprintf (t, "%s-set-position", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set-position"));
  sprintf (t, "%s-get-position", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-position"));
  sprintf (t, "%s-remove", s);
  gh_define (t, scm_symbol_value0("fltk-browser-remove"));
  sprintf (t, "%s-show", s);
  gh_define (t, scm_symbol_value0("fltk-browser-show"));
  sprintf (t, "%s-get-size", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-size"));
  sprintf (t, "%s-set_text", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set_text"));
  sprintf (t, "%s-get-text", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-text"));
  sprintf (t, "%s-get-topline", s);
  gh_define (t, scm_symbol_value0("fltk-browser-get-topline"));
  sprintf (t, "%s-set-topline", s);
  gh_define (t, scm_symbol_value0("fltk-browser-set-topline"));
  sprintf (t, "%s-visible", s);
  gh_define (t, scm_symbol_value0("fltk-browser-visible"));
}
