// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Browser_.H>
#include "guile_fltk.h"

long fltk_browser_base_tag;

//***************************************************************************
//   
//  Fl_Browser_ encapsulation method(s):
//
//***************************************************************************

static SCM
fltk_browser_base_has_scrollbar (SCM widget_smob, SCM a)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-base-has-scrollbar");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-base-has-scrollbar");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  widget->has_scrollbar(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_base_set_textcolor (SCM widget_smob, SCM a)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-textcolor");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-base-set-textcolor");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  widget->textcolor(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_base_get_textcolor (SCM widget_smob)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-textcolor");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->textcolor());
}

static SCM
fltk_browser_base_set_textfont (SCM widget_smob, SCM a)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-textfont");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-base-set-textfont");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  widget->textfont(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_base_get_textfont (SCM widget_smob)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-textfont");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->textfont());
}

static SCM
fltk_browser_base_set_textsize (SCM widget_smob, SCM a)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-set-textsize");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-base-set-textsize");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  widget->textsize(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_browser_base_get_textsize (SCM widget_smob)
{
  Fl_Browser_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_base_tag),
              widget_smob, SCM_ARG1, "fltk-browser-get-textsize");

  widget = (Fl_Browser_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->textsize());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Browser_ *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Browser_) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Browser_ *widget = (Fl_Browser_ *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Browser_ ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-browser-base scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_browser_base_init ()
{
  fltk_browser_base_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-browser-base", fltk_browser_base_tag);

  // add new methods
  gh_new_procedure2_0 ("fltk-browser-base-has-scrollbar", 
		       fltk_browser_base_has_scrollbar);
  gh_new_procedure2_0 ("fltk-browser-base-set-textcolor", 
		       fltk_browser_base_set_textcolor);
  gh_new_procedure1_0 ("fltk-browser-base-get-textcolor", 
		       fltk_browser_base_get_textcolor);
  gh_new_procedure2_0 ("fltk-browser-base-set-textfont", 
		       fltk_browser_base_set_textfont);
  gh_new_procedure1_0 ("fltk-browser-base-get-textfont", 
		       fltk_browser_base_get_textfont);
  gh_new_procedure2_0 ("fltk-browser-base-set-textsize", 
		       fltk_browser_base_set_textsize);
  gh_new_procedure1_0 ("fltk-browser-base-get-textsize", 
		       fltk_browser_base_get_textsize);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_browser_base_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_browser_base_tag);
  fltk_widget_subclass(s, tag);
   
  sprintf (t, "%s-has-scrollbar", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-has-scrollbar"));
  sprintf (t, "%s-set-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-set-textcolor"));
  sprintf (t, "%s-get-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-get-textcolor"));
  sprintf (t, "%s-set-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-set-textfont"));
  sprintf (t, "%s-get-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-get-textfont"));
  sprintf (t, "%s-set-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-set-textsize"));
  sprintf (t, "%s-get-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-browser-base-get-textsize"));
}
