// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Button.H>
#include "guile_fltk.h"

long fltk_button_tag;

//***************************************************************************
//   
//  Fl_Button encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_button_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Button * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-button-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-button-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-button-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-button-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-button-new");

  widget = new Fl_Button(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_button_tag);
  gfltk_register(widget_smob);

  return widget_smob;
}

static SCM
fltk_button_clear (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-clear");

  widget = (Fl_Button *) gh_cdr (widget_smob);
  widget->clear();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_set_down_box (SCM widget_smob, SCM a)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-set-down-box");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-button-set-down-box");
  
  widget = (Fl_Button *) gh_cdr (widget_smob);
  int x = SCM_NFALSEP(a);
  widget->down_box((Fl_Boxtype)x);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_get_down_box (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-get-down-box");

  widget = (Fl_Button *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->down_box());
}

static SCM
fltk_button_set (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-set");

  widget = (Fl_Button *) gh_cdr (widget_smob);
  widget->set();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_setonly (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-setonly");

  widget = (Fl_Button *) gh_cdr (widget_smob);
  widget->setonly();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_set_shortcut (SCM widget_smob, SCM a)
{
  Fl_Button *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-set-shortcut");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-button-set-shortcut");
  
  widget = (Fl_Button *) gh_cdr (widget_smob);
  int x = SCM_NFALSEP(a);
  widget->shortcut(x);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_get_shortcut (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-get-shortcut");

  widget = (Fl_Button *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->shortcut());
}

static SCM
fltk_button_set_type (SCM widget_smob, SCM a)
{
  Fl_Button *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-set-type");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-button-set-type");
  
  widget = (Fl_Button *) gh_cdr (widget_smob);
  int x = SCM_NFALSEP(a);
  widget->type(x);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_get_type (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-get-type");

  widget = (Fl_Button *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->type());
}

static SCM
fltk_button_set_value (SCM widget_smob, SCM a)
{
  Fl_Button *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-set-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-button-set-value");
  
  widget = (Fl_Button *) gh_cdr (widget_smob);
  int x = SCM_NFALSEP(a);
  widget->value(x);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_get_value (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-get-value");

  widget = (Fl_Button *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->value());
}

static SCM
fltk_button_set_when (SCM widget_smob, SCM a)
{
  Fl_Button *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-set-when");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-button-set-when");
  
  widget = (Fl_Button *) gh_cdr (widget_smob);
  int aa = SCM_NFALSEP(a);
  widget->when(aa);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_button_get_when (SCM widget_smob)
{
  Fl_Button *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_button_tag),
              widget_smob, SCM_ARG1, "fltk-button-get-when");

  widget = (Fl_Button *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->when());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Button *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Button) + sizeof (Fl_Button));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Button *widget = (Fl_Button *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Button ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk_box scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_button_init ()
{
  fltk_button_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-button", fltk_button_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-button-new",
		       fltk_button_new);
  gh_new_procedure1_0 ("fltk-button-clear",
		       fltk_button_clear);
  gh_new_procedure2_0 ("fltk-button-set-down-box",
		       fltk_button_set_down_box);
  gh_new_procedure1_0 ("fltk-button-get-down-box",
		       fltk_button_get_down_box);
  gh_new_procedure1_0 ("fltk-button-set",
		       fltk_button_set);
  gh_new_procedure1_0 ("fltk-button-setonly",
		       fltk_button_setonly);
  gh_new_procedure2_0 ("fltk-button-set-shortcut",
		       fltk_button_set_shortcut);
  gh_new_procedure1_0 ("fltk-button-get-shortcut",
		       fltk_button_get_shortcut);
  gh_new_procedure2_0 ("fltk-button-set-type",
		       fltk_button_set_type);
  gh_new_procedure1_0 ("fltk-button-get-type",
		       fltk_button_get_type);
  gh_new_procedure2_0 ("fltk-button-set-value",
		       fltk_button_set_value);
  gh_new_procedure1_0 ("fltk-button-get-value",
		       fltk_button_get_value);
  gh_new_procedure2_0 ("fltk-button-set-when",
		       fltk_button_set_when);
  gh_new_procedure1_0 ("fltk-button-get-when",
		       fltk_button_get_when);
  gh_define ("FL-TOGGLE-BUTTON", gh_long2scm(FL_TOGGLE_BUTTON));
  gh_define ("FL-RADIO-BUTTON", gh_long2scm(FL_RADIO_BUTTON));
  gh_define ("FL-HIDDEN-BUTTON", gh_long2scm(FL_HIDDEN_BUTTON));

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_button_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_button_tag);
  fltk_widget_subclass(s, tag);

  sprintf(t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-button-new"));
  sprintf(t, "%s-clear", s);
  gh_define (t, scm_symbol_value0("fltk-button-clear"));
  sprintf(t, "%s-set-down-box", s);
  gh_define (t, scm_symbol_value0("fltk-button-set-down-box"));
  sprintf(t, "%s-get-down-box", s);
  gh_define (t, scm_symbol_value0("fltk-button-get-down-box"));
  sprintf(t, "%s-set", s);
  gh_define (t, scm_symbol_value0("fltk-button-set"));
  sprintf(t, "%s-setonly", s);
  gh_define (t, scm_symbol_value0("fltk-button-setonly"));
  sprintf(t, "%s-set-shortcut", s);
  gh_define (t, scm_symbol_value0("fltk-button-set-shortcut"));
  sprintf(t, "%s-get-shortcut", s);
  gh_define (t, scm_symbol_value0("fltk-button-get-shortcut"));
  sprintf(t, "%s-set-type", s);
  gh_define (t, scm_symbol_value0("fltk-button-set-type"));
  sprintf(t, "%s-get-type", s);
  gh_define (t, scm_symbol_value0("fltk-button-get-type"));
  sprintf(t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-button-set-value"));
  sprintf(t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-button-get-value"));
  sprintf(t, "%s-set-when", s);
  gh_define (t, scm_symbol_value0("fltk-button-set-when"));
  sprintf(t, "%s-get-when", s);
  gh_define (t, scm_symbol_value0("fltk-button-get-when"));
}
