// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Chart.H>
#include "guile_fltk.h"

long fltk_chart_tag;

//***************************************************************************
//   
//  Fl_Chart encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_chart_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Chart * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-chart-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-chart-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-chart-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-chart-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-chart-new");

  widget = new Fl_Chart(gh_scm2int(x), gh_scm2int(y), 
			  gh_scm2int(w), gh_scm2int(h),
			  gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_chart_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}


static SCM
fltk_chart_add (SCM widget_smob, SCM a, SCM b, SCM c)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-add");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-chart-add");
  SCM_ASSERT (gh_string_p(b), b, SCM_ARG3, "fltk-chart-add");
  SCM_ASSERT (gh_number_p(c), c, SCM_ARG4, "fltk-chart-add");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->add(gh_scm2double(a), gh_scm2newstr(b, NULL), gh_scm2int(c));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_get_autosize (SCM widget_smob)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-get-autosize");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  if (widget->autosize())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_chart_set_autosize (SCM widget_smob, SCM a)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-set-autosize");
  SCM_ASSERT ((SCM_NIMP (a)), a, SCM_ARG2, "fltk-chart-add");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->autosize((uchar)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_bounds (SCM widget_smob, SCM a, SCM b)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-set-autosize");
  SCM_ASSERT (gh_number_p (a), a, SCM_ARG2, "fltk-chart-set-autosize");
  SCM_ASSERT (gh_number_p (b), b, SCM_ARG3, "fltk-chart-set-autosize");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->bounds(gh_scm2double(a), gh_scm2double(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_clear (SCM widget_smob)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-set-autosize");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->clear();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_insert (SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-insert");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-chart-insert");
  SCM_ASSERT (gh_number_p(b), b, SCM_ARG3, "fltk-chart-insert");
  SCM_ASSERT (gh_string_p(c), c, SCM_ARG4, "fltk-chart-insert");
  SCM_ASSERT (gh_number_p(d), d, SCM_ARG5, "fltk-chart-insert");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->insert(gh_scm2int(a), gh_scm2double(b), gh_scm2newstr(c, NULL), 
    (uchar)gh_scm2int(d));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_get_maxsize (SCM widget_smob)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-get-maxsize");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  return gh_int2scm(widget->maxsize());
}

static SCM
fltk_chart_set_maxsize (SCM widget_smob, SCM a)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-set-maxsize");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-chart-set-maxsize");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);

  widget->maxsize(aa);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_replace (SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-replace");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-chart-replace");
  SCM_ASSERT (gh_number_p(b), b, SCM_ARG3, "fltk-chart-replace");
  SCM_ASSERT (gh_string_p(c), c, SCM_ARG4, "fltk-chart-replace");
  SCM_ASSERT (gh_number_p(d), d, SCM_ARG5, "fltk-chart-replace");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->replace(gh_scm2int(a), gh_scm2double(b), gh_scm2newstr(c, NULL), (uchar)gh_scm2int(d));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_chart_size (SCM widget_smob)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-size");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  return gh_int2scm(widget->size());
}

static SCM
fltk_chart_get_type (SCM widget_smob)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-get-type");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->type());
}

static SCM
fltk_chart_set_type (SCM widget_smob, SCM a)
{
  Fl_Chart *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_chart_tag),
              widget_smob, SCM_ARG1, "fltk-chart-set-type");
  SCM_ASSERT ((SCM_NIMP (a)), a, SCM_ARG2, "fltk-chart-set-type");

  widget = (Fl_Chart *) gh_cdr (widget_smob);

  widget->type((uchar)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Chart *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Chart) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Chart *widget = (Fl_Chart *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Chart ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-chart scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_chart_init ()
{
  fltk_chart_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-chart", fltk_chart_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-chart-new", 
		       fltk_chart_new);
  gh_new_procedure4_0 ("fltk-chart-add", 
		       fltk_chart_add);
  gh_new_procedure1_0 ("fltk-chart-get-autosize", 
		       fltk_chart_get_autosize);
  gh_new_procedure2_0 ("fltk-chart-set-autosize", 
		       fltk_chart_set_autosize);
  gh_new_procedure3_0 ("fltk-chart-bounds", 
		       fltk_chart_bounds);
  gh_new_procedure1_0 ("fltk-chart-clear", 
		       fltk_chart_clear);
  gh_new_procedure5_0 ("fltk-chart-insert", 
		       fltk_chart_insert);
  gh_new_procedure1_0 ("fltk-chart-get-maxsize", 
		       fltk_chart_get_maxsize);
  gh_new_procedure2_0 ("fltk-chart-set-maxsize", 
		       fltk_chart_set_maxsize);
  gh_new_procedure5_0 ("fltk-chart-replace", 
		       fltk_chart_replace);
  gh_new_procedure1_0 ("fltk-chart-size", 
		       fltk_chart_size);
  gh_new_procedure1_0 ("fltk-chart-get-type", 
		       fltk_chart_get_type);
  gh_new_procedure2_0 ("fltk-chart-set-type", 
		       fltk_chart_set_type);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_chart_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_chart_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-chart-new"));
  sprintf (t, "%s-add", s);
  gh_define (t, scm_symbol_value0("fltk-chart-add"));
  sprintf (t, "%s-get-autosize", s);
  gh_define (t, scm_symbol_value0("fltk-chart-get-autosize"));
  sprintf (t, "%s-set-autosize", s);
  gh_define (t, scm_symbol_value0("fltk-chart-set-autosize"));
  sprintf (t, "%s-bounds", s);
  gh_define (t, scm_symbol_value0("fltk-chart-bounds"));
  sprintf (t, "%s-clear", s);
  gh_define (t, scm_symbol_value0("fltk-chart-clear"));
  sprintf (t, "%s-insert", s);
  gh_define (t, scm_symbol_value0("fltk-chart-insert"));
  sprintf (t, "%s-get-maxsize", s);
  gh_define (t, scm_symbol_value0("fltk-chart-get-maxsize"));
  sprintf (t, "%s-set-maxsize", s);
  gh_define (t, scm_symbol_value0("fltk-chart-set-maxsize"));
  sprintf (t, "%s-replace", s);
  gh_define (t, scm_symbol_value0("fltk-chart-replace"));
  sprintf (t, "%s-get-type", s);
  gh_define (t, scm_symbol_value0("fltk-chart-get-type"));
  sprintf (t, "%s-set-type", s);
  gh_define (t, scm_symbol_value0("fltk-chart-set-type"));

}
