// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Choice.H>
#include "guile_fltk.h"

long fltk_choice_tag;

//***************************************************************************
//   
//  Fl_Choice encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_choice_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Choice * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-choice-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-choice-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-choice-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-choice-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-choice-new");

  widget = new Fl_Choice(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_choice_tag);
  gfltk_register(widget_smob);

  return widget_smob;
}

static SCM
fltk_choice_get_value (SCM widget_smob)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-get-value");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->value());
}

static SCM
fltk_choice_set_value (SCM widget_smob, SCM a)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-set-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-choice-set-value");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  widget->value(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_choice_changed (SCM widget_smob)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-changed");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  if (widget->changed())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_choice_set_changed (SCM widget_smob)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-set-changed");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  widget->set_changed();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_choice_clear_changed (SCM widget_smob)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-clear-changed");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  widget->clear_changed();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_choice_get_down_box (SCM widget_smob)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-get-down-box");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->down_box());
}

static SCM
fltk_choice_set_down_box (SCM widget_smob, SCM a)
{
  Fl_Choice *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_choice_tag),
              widget_smob, SCM_ARG1, "fltk-choice-set-down-box");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-choice-set-down-box");

  widget = (Fl_Choice *) gh_cdr (widget_smob);

  widget->down_box((Fl_Boxtype)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}


//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Choice *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Gfltk) + sizeof (Fl_Choice));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Choice *widget = (Fl_Choice *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Choice ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-check-button scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_choice_init ()
{
  fltk_choice_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_menu_subclass("fltk-choice", fltk_choice_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-choice-new",
		       fltk_choice_new);
  gh_new_procedure1_0 ("fltk-choice-get-value",
		       fltk_choice_get_value);
  gh_new_procedure2_0 ("fltk-choice-set-value",
		       fltk_choice_set_value);
  gh_new_procedure1_0 ("fltk-choice-changed",
		       fltk_choice_changed);
  gh_new_procedure1_0 ("fltk-choice-set-changed",
		       fltk_choice_set_changed);
  gh_new_procedure1_0 ("fltk-choice-clear-changed",
		       fltk_choice_clear_changed);
  gh_new_procedure1_0 ("fltk-choice-get-down-box",
		       fltk_choice_get_down_box);
  gh_new_procedure2_0 ("fltk-choice-set-down-box",
		       fltk_choice_set_down_box);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_choice_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_choice_tag);
  fltk_menu_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-choice-new"));
  sprintf (t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-choice-get-value"));
  sprintf (t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-choice-set-value"));
  sprintf (t, "%s-changed", s);
  gh_define (t, scm_symbol_value0("fltk-choice-changed"));
  sprintf (t, "%s-set-changed", s);
  gh_define (t, scm_symbol_value0("fltk-choice-set-changed"));
  sprintf (t, "%s-clear-changed", s);
  gh_define (t, scm_symbol_value0("fltk-choice-clear-changed"));
  sprintf (t, "%s-get-down-box", s);
  gh_define (t, scm_symbol_value0("fltk-choice-get-down-box"));
  sprintf (t, "%s-set-down-box", s);
  gh_define (t, scm_symbol_value0("fltk-choice-set-down-box"));
}
