// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Clock.H>
#include "guile_fltk.h"

long fltk_clock_tag;

//***************************************************************************
//   
//  Fl_Clock encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_clock_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Clock * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-clock-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-clock-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-clock-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-clock-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-clock-new");

  widget = new Fl_Clock(gh_scm2int(x), gh_scm2int(y), 
			gh_scm2int(w), gh_scm2int(h),
			gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_clock_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_clock_hour(SCM widget_smob)
{
  Fl_Clock *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_clock_tag),
              widget_smob, SCM_ARG1, "fltk-clock-hour");

  widget = (Fl_Clock *) gh_cdr (widget_smob);

  return gh_int2scm(widget->hour());
}

static SCM
fltk_clock_minute(SCM widget_smob)
{
  Fl_Clock *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_clock_tag),
              widget_smob, SCM_ARG1, "fltk-clock-minute");

  widget = (Fl_Clock *) gh_cdr (widget_smob);

  return gh_int2scm(widget->minute());
}

static SCM
fltk_clock_second(SCM widget_smob)
{
  Fl_Clock *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_clock_tag),
              widget_smob, SCM_ARG1, "fltk-clock-second");

  widget = (Fl_Clock *) gh_cdr (widget_smob);
  
  return gh_int2scm(widget->second());
}

static SCM
fltk_clock_value(SCM widget_smob, SCM a, SCM b, SCM c)
{
  Fl_Clock *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_clock_tag),
              widget_smob, SCM_ARG1, "fltk-clock-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-clock-value");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-clock-value");
  SCM_ASSERT (gh_number_p(c), c,  SCM_ARG4, "fltk-clock-value");

  widget = (Fl_Clock *) gh_cdr (widget_smob);
  widget->value(gh_scm2int(a), gh_scm2int(b), gh_scm2int(c));

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Clock *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Clock) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Clock *widget = (Fl_Clock *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Clock ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-clock scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_clock_init ()
{
  fltk_clock_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-clock", fltk_clock_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-clock-new", 
		       fltk_clock_new);
  gh_new_procedure1_0 ("fltk-clock-hour", 
		       fltk_clock_hour);
  gh_new_procedure1_0 ("fltk-clock-minute", 
		       fltk_clock_minute);
  gh_new_procedure1_0 ("fltk-clock-second", 
		       fltk_clock_second);
  gh_new_procedure4_0 ("fltk-clock-value", 
		       fltk_clock_value);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_clock_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_clock_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-clock-new"));
  sprintf (t, "%s-hour", s);
  gh_define (t, scm_symbol_value0("fltk-clock-hour"));
  sprintf (t, "%s-minute", s);
  gh_define (t, scm_symbol_value0("fltk-clock-minute"));
  sprintf (t, "%s-second", s);
  gh_define (t, scm_symbol_value0("fltk-clock-second"));
  sprintf (t, "%s-value", s);
  gh_define (t, scm_symbol_value0("fltk-clock-value"));
}
