// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Color_Chooser.H>
#include "guile_fltk.h"

long fltk_color_chooser_tag;

//***************************************************************************
//   
//  Fl_Color_Chooser encapsulation method(s):
//
//***************************************************************************


static SCM 
fltk_color_chooser_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Color_Chooser * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-color_chooser-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-color_chooser-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-color_chooser-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-color_chooser-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-color-chooser-new");

  widget = new Fl_Color_Chooser(gh_scm2int(x), gh_scm2int(y), 
				gh_scm2int(w), gh_scm2int(h),
				gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_color_chooser_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_color_chooser_hue (SCM widget_smob)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-resize");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_double2scm(widget->hue());
}

static SCM
fltk_color_chooser_saturation (SCM widget_smob)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-resize");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_double2scm(widget->saturation());
}

static SCM
fltk_color_chooser_value (SCM widget_smob)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-resize");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_double2scm(widget->value());
}

static SCM
fltk_color_chooser_r (SCM widget_smob)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-r");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_double2scm(widget->r());
}

static SCM
fltk_color_chooser_g (SCM widget_smob)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-g");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_double2scm(widget->g());
}

static SCM
fltk_color_chooser_b (SCM widget_smob)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-b");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_double2scm(widget->b());
}

static SCM
fltk_color_chooser_rgb (SCM widget_smob, SCM a, SCM b, SCM c)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-rgb");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-color-chooser-rgb");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-color-chooser-rgb");
  SCM_ASSERT (gh_number_p(c), c,  SCM_ARG4, "fltk-color-chooser-rgb");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_int2scm(widget->rgb(gh_scm2double(a), 
				gh_scm2double(b),
				gh_scm2double(c)));
}

static SCM
fltk_color_chooser_hsv (SCM widget_smob, SCM a, SCM b, SCM c)
{
  Fl_Color_Chooser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_color_chooser_tag),
              widget_smob, SCM_ARG1, "fltk-color_chooser-hsv");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-color-chooser-hsv");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-color-chooser-hsv");
  SCM_ASSERT (gh_number_p(c), c,  SCM_ARG4, "fltk-color-chooser-hsv");

  widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  return gh_int2scm(widget->hsv(gh_scm2double(a), 
				gh_scm2double(b),
				gh_scm2double(c)));
}

static SCM
fltk_color_chooser_hsv2rgb (SCM a, SCM b, SCM c)
{
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG1, "fltk-color-chooser-hsv2rgb");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG2, "fltk-color-chooser-hsv2rgb");
  SCM_ASSERT (gh_number_p(c), c,  SCM_ARG3, "fltk-color-chooser-hsv2rgb");

  double d, e, f;

  Fl_Color_Chooser::hsv2rgb(gh_scm2double(a), gh_scm2double(b),
			    gh_scm2double(c), d, e, f);

  return gh_cons(gh_double2scm(d), 
		 gh_cons(gh_double2scm(e), 
			 gh_cons(gh_double2scm(f), SCM_EOL)));
}

static SCM
fltk_color_chooser_rgb2hsv (SCM a, SCM b, SCM c)
{
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG1, "fltk-color-chooser-rgb2hsv");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG2, "fltk-color-chooser-rgb2hsv");
  SCM_ASSERT (gh_number_p(c), c,  SCM_ARG3, "fltk-color-chooser-rgb2hsv");

  double d, e, f;

  Fl_Color_Chooser::rgb2hsv(gh_scm2double(a), gh_scm2double(b),
		  gh_scm2double(c), d, e, f);

  return gh_cons(gh_double2scm(d), 
		 gh_cons(gh_double2scm(e), 
			 gh_cons(gh_double2scm(f), SCM_EOL)));
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Color_Chooser *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Color_Chooser) + sizeof (Fl_Color_Chooser));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Color_Chooser *widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  scm_puts ("#<widget Fl_Color_Chooser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);

  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-color_chooser scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_color_chooser_init ()
{
  fltk_color_chooser_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_group_subclass("fltk-color_chooser", fltk_color_chooser_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-color_chooser-new",
		       fltk_color_chooser_new);
  gh_new_procedure1_0 ("fltk-color_chooser-hue",
		       fltk_color_chooser_hue);
  gh_new_procedure1_0 ("fltk-color_chooser-saturation",
		       fltk_color_chooser_saturation);
  gh_new_procedure1_0 ("fltk-color_chooser-value",
		       fltk_color_chooser_value);
  gh_new_procedure1_0 ("fltk-color_chooser-r",
		       fltk_color_chooser_r);
  gh_new_procedure1_0 ("fltk-color_chooser-g",
		       fltk_color_chooser_g);
  gh_new_procedure1_0 ("fltk-color_chooser-b",
		       fltk_color_chooser_b);
  gh_new_procedure4_0 ("fltk-color_chooser-rgb",
		       fltk_color_chooser_rgb);
  gh_new_procedure4_0 ("fltk-color_chooser-hsv",
		       fltk_color_chooser_hsv);
  gh_new_procedure3_0 ("fltk-color_chooser-rgb2hsv",
		       fltk_color_chooser_rgb2hsv);
  gh_new_procedure3_0 ("fltk-color_chooser-hsv2rgb",
		       fltk_color_chooser_hsv2rgb);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_color_chooser_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_color_chooser_tag);
  fltk_button_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-new"));
  sprintf (t, "%s-hue", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-hue"));
  sprintf (t, "%s-saturation", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-saturation"));
  sprintf (t, "%s-value", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-value"));
  sprintf (t, "%s-r", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-r"));
  sprintf (t, "%s-g", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-g"));
  sprintf (t, "%s-b", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-b"));
  sprintf (t, "%s-rgb", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-rgb"));
  sprintf (t, "%s-hsv", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-hsv"));
  sprintf (t, "%s-rgb2hsv", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-rgb2hsv"));
  sprintf (t, "%s-hsv2rgb", s);
  gh_define (t, scm_symbol_value0("fltk-color_chooser-hsv2rgb"));

}
