// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Dial.H>
#include "guile_fltk.h"

long fltk_dial_tag;

//***************************************************************************
//   
//  Fl_Dial encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_dial_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Dial * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-dial-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-dial-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-dial-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-dial-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-dial-new");

  widget = new Fl_Dial(gh_scm2int(x), gh_scm2int(y), 
		       gh_scm2int(w), gh_scm2int(h),
		       gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_dial_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_dial_angles(SCM widget_smob, SCM a, SCM b)
{
  Fl_Dial *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_dial_tag),
              widget_smob, SCM_ARG1, "fltk-dial-angles");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-dial-angles");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-dial-angles");

  widget = (Fl_Dial *) gh_cdr (widget_smob);

  widget->angles(gh_scm2double(a), gh_scm2double(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_dial_type(SCM widget_smob, SCM a)
{
  Fl_Dial *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_dial_tag),
              widget_smob, SCM_ARG1, "fltk-dial-type");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-dial-type");

  widget = (Fl_Dial *) gh_cdr (widget_smob);

  widget->type((uchar)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Dial *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Dial) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Dial *widget = (Fl_Dial *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Dial ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-dial scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_dial_init ()
{
  fltk_dial_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_valuator_subclass("fltk-dial", fltk_dial_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-dial-new", 
		       fltk_dial_new);
  gh_new_procedure3_0 ("fltk-dial-angles", 
		       fltk_dial_angles);
  gh_new_procedure2_0 ("fltk-dial-type", 
		       fltk_dial_type);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_dial_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_dial_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-dial-new"));
  sprintf (t, "%s-type", s);
  gh_define (t, scm_symbol_value0("fltk-dial-type"));
  sprintf (t, "%s-angles", s);
  gh_define (t, scm_symbol_value0("fltk-dial-angles"));

}
