// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl.H>
#include "guile_fltk.h"
#include "guile_support.h"

static SCM
fltk_run ()
{
  Fl::run();
  return SCM_UNSPECIFIED;
}

static SCM
fltk_event_clicks ()
{
  return gh_long2scm(Fl::event_clicks());
}

static SCM
fltk_beep ()
{
  printf("\007"); 
  fflush(stdout);
  return SCM_UNSPECIFIED;
}

static SCM
fltk_test (SCM a)
{
  while (gh_car(a) != SCM_EOL)
    {
      SCM option = gh_car(a);
      a = gh_cdr(a);
      if (gh_car(a) == SCM_EOL)
	break;

      SCM argument = gh_car(a);
      a = gh_cdr(a);

      //SCM_ASSERT (SCM_SYMBOLP(option), option, SCM_ARGn, "fltk-test");

      printf("option=%s\n", SCM_CHARS(option));

      gh_display(argument);
      printf("\n");
      if (a == SCM_EOL)
	break;
    }
   return SCM_UNSPECIFIED;
}

void
fltk_fl_init ()
{
  gh_new_procedure0_0 ("fltk-run",
		       fltk_run);
  gh_new_procedure0_0 ("fltk-event-clicks",
		       fltk_event_clicks);
  gh_new_procedure0_0 ("fltk-beep",
		       fltk_beep);
  gh_new_procedure0_0_1 ("fltk-test",
		       fltk_test);

  // Fltk Enumerations
  gh_define ("FL-WHEN-NEVER", gh_long2scm(FL_WHEN_NEVER));
  gh_define ("FL-WHEN-CHANGED", gh_long2scm(FL_WHEN_CHANGED));
  gh_define ("FL-WHEN-RELEASE", gh_long2scm(FL_WHEN_RELEASE));
  gh_define ("FL-WHEN-RELEASE-ALWAYS", gh_long2scm(FL_WHEN_RELEASE_ALWAYS));
  gh_define ("FL-WHEN-ENTER-KEY", gh_long2scm(FL_WHEN_ENTER_KEY));
  gh_define ("FL-WHEN-ENTER-KEY-ALWAYS", gh_long2scm(FL_WHEN_ENTER_KEY_ALWAYS));
  gh_define ("FL-WHEN-NOT-CHANGED", gh_long2scm(FL_WHEN_NOT_CHANGED));

  gh_define ("FL-SHIFT", gh_long2scm(FL_SHIFT));
  gh_define ("FL-CAPS-LOCK", gh_long2scm(FL_CAPS_LOCK));
  gh_define ("FL-CTRL", gh_long2scm(FL_CTRL));
  gh_define ("FL-ALT", gh_long2scm(FL_ALT));
  gh_define ("FL-NUM-LOCK", gh_long2scm(FL_NUM_LOCK));
  gh_define ("FL-META", gh_long2scm(FL_META));
  gh_define ("FL-SCROLL-LOCK", gh_long2scm(FL_SCROLL_LOCK));
  gh_define ("FL-BUTTON1", gh_long2scm(FL_BUTTON1));
  gh_define ("FL-BUTTON2", gh_long2scm(FL_BUTTON2));
  gh_define ("FL-BUTTON3", gh_long2scm(FL_BUTTON3));

  gh_define ("Fl-Button", gh_long2scm(FL_Button));
  gh_define ("Fl-Backspace", gh_long2scm(FL_BackSpace));
  gh_define ("Fl-Tab", gh_long2scm(FL_Tab));
  gh_define ("Fl-Enter", gh_long2scm(FL_Enter));
  gh_define ("Fl-Pause", gh_long2scm(FL_Pause));
  gh_define ("Fl-Scroll-Lock", gh_long2scm(FL_Scroll_Lock));
  gh_define ("Fl-Escape", gh_long2scm(FL_Escape));
  gh_define ("Fl-Home", gh_long2scm(FL_Home));
  gh_define ("Fl-Left", gh_long2scm(FL_Left));
  gh_define ("Fl-Up", gh_long2scm(FL_Up));
  gh_define ("Fl-Right", gh_long2scm(FL_Right));
  gh_define ("Fl-Down", gh_long2scm(FL_Down));
  gh_define ("Fl-Page-Up", gh_long2scm(FL_Page_Up));
  gh_define ("Fl-Page-Down", gh_long2scm(FL_Page_Down));
  gh_define ("Fl-End", gh_long2scm(FL_End));
  gh_define ("Fl-Print", gh_long2scm(FL_Print));
  gh_define ("Fl-Insert", gh_long2scm(FL_Insert));
  gh_define ("Fl-Menu", gh_long2scm(FL_Menu));
  gh_define ("Fl-Num_Lock", gh_long2scm(FL_Num_Lock));
  gh_define ("Fl-KP", gh_long2scm(FL_KP));
  gh_define ("Fl-KP-Enter", gh_long2scm(FL_KP_Enter));
  gh_define ("Fl-KP-Last", gh_long2scm(FL_KP_Last));

  // Enumerations:
  gh_define ("FL-MAJOR-VERSION", gh_long2scm(FL_MAJOR_VERSION));
  gh_define ("FL-MINOR-VERSION", gh_long2scm(FL_MINOR_VERSION));
  gh_define ("FL-PATCH-VERSION", gh_long2scm(FL_PATCH_VERSION));
  gh_define ("FL-NO-EVENT", gh_long2scm(FL_NO_EVENT));
  gh_define ("FL-PUSH", gh_long2scm(FL_PUSH));
  gh_define ("FL-RELEASE", gh_long2scm(FL_RELEASE));
  gh_define ("FL-ENTER", gh_long2scm(FL_ENTER));
  gh_define ("FL-LEAVE", gh_long2scm(FL_LEAVE));
  gh_define ("FL-DRAG", gh_long2scm(FL_DRAG));
  gh_define ("FL-FOCUS", gh_long2scm(FL_FOCUS));
  gh_define ("FL-UNFOCUS", gh_long2scm(FL_UNFOCUS));
  gh_define ("FL-KEYBOARD", gh_long2scm(FL_KEYBOARD));
  gh_define ("FL-CLOSE", gh_long2scm(FL_CLOSE));
  gh_define ("FL-MOVE", gh_long2scm(FL_MOVE));
  gh_define ("FL-SHORTCUT", gh_long2scm(FL_SHORTCUT));
  gh_define ("FL-DEACTIVATE", gh_long2scm(FL_DEACTIVATE));
  gh_define ("FL-ACTIVATE", gh_long2scm(FL_ACTIVATE));
  gh_define ("FL-HIDE", gh_long2scm(FL_HIDE));
  gh_define ("FL-SHOW", gh_long2scm(FL_SHOW));
  gh_define ("FL-PASTE", gh_long2scm(FL_PASTE));
  gh_define ("FL-SELECTIONCLEAR", gh_long2scm(FL_SELECTIONCLEAR));
  
  gh_define ("FL-NO-BOX", gh_long2scm(FL_NO_BOX));
  gh_define ("FL-FLAT-BOX", gh_long2scm(FL_FLAT_BOX));
  gh_define ("FL-UP-BOX", gh_long2scm(FL_UP_BOX)); 
  gh_define ("FL-DOWN-BOX", gh_long2scm(FL_DOWN_BOX)); 
  gh_define ("FL-UP-FRAME", gh_long2scm(FL_UP_FRAME)); 
  gh_define ("FL-DOWN-FRAME", gh_long2scm(FL_DOWN_FRAME)); 
  
  gh_define ("FL-THIN-UP-BOX", gh_long2scm(FL_THIN_UP_BOX)); 
  gh_define ("FL-THIN-DOWN-BOX", gh_long2scm(FL_THIN_DOWN_BOX)); 
  gh_define ("FL-THIN-UP-FRAME", gh_long2scm(FL_THIN_UP_FRAME)); 
  gh_define ("FL-THIN-DOWN-FRAME", gh_long2scm(FL_THIN_DOWN_FRAME)); 
  
  gh_define ("FL-ENGRAVED-BOX", gh_long2scm(FL_ENGRAVED_BOX)); 
  gh_define ("FL-EMBOSSED-BOX", gh_long2scm(FL_EMBOSSED_BOX)); 
  gh_define ("FL-ENGRAVED-FRAME", gh_long2scm(FL_ENGRAVED_FRAME)); 
  gh_define ("FL-EMBOSSED-FRAME", gh_long2scm(FL_EMBOSSED_FRAME)); 
  gh_define ("FL-BORDER-BOX", gh_long2scm(FL_BORDER_BOX)); 
  gh_define ("FL-BORDER-FRAME", gh_long2scm(FL_BORDER_FRAME)); 
  gh_define ("FL-SHADOW-BOX", gh_long2scm(FL_SHADOW_BOX)); 
  gh_define ("FL-SHADOW-FRAME", gh_long2scm(FL_SHADOW_FRAME)); 
  gh_define ("FL-ROUNDED-BOX", gh_long2scm(FL_ROUNDED_BOX)); 
  gh_define ("FL-ROUNDED-FRAME", gh_long2scm(FL_ROUNDED_FRAME)); 
  gh_define ("FL-RSHADOW-BOX", gh_long2scm(FL_RSHADOW_BOX)); 
  gh_define ("FL-RFLAT-BOX", gh_long2scm(FL_RFLAT_BOX)); 
  gh_define ("FL-ROUND-DOWN-BOX", gh_long2scm(FL_ROUND_DOWN_BOX));    
  gh_define ("FL-ROUND-UP-BOX", gh_long2scm(FL_ROUND_UP_BOX));    
  gh_define ("FL-DIAMOND-DOWN-BOX", gh_long2scm(FL_DIAMOND_DOWN_BOX));    
  gh_define ("FL-DIAMOND-UP-BOX", gh_long2scm(FL_DIAMOND_UP_BOX));    
  gh_define ("FL-OVAL-BOX", gh_long2scm(FL_OVAL_BOX)); 
  gh_define ("FL-OVAL-FRAME", gh_long2scm(FL_OVAL_FRAME)); 
  gh_define ("FL-OSHADOW-BOX", gh_long2scm(FL_OSHADOW_BOX)); 
  gh_define ("FL-OFLAT-BOX", gh_long2scm(FL_OFLAT_BOX)); 
  gh_define ("FL-FREE-BOXTYPE", gh_long2scm(FL_FREE_BOXTYPE)); 
   
  gh_define ("FL-NORMAL-LABEL", gh_long2scm(FL_NORMAL_LABEL)); 
  gh_define ("FL-NO-LABEL", gh_long2scm(FL_NO_LABEL)); 
  gh_define ("FL-SYMBOL-LABEL", gh_long2scm(FL_SYMBOL_LABEL)); 
  gh_define ("FL-SHADOW-LABEL", gh_long2scm(FL_SHADOW_LABEL)); 
  gh_define ("FL-ENGRAVED-LABEL", gh_long2scm(FL_ENGRAVED_LABEL)); 
  gh_define ("FL-EMBOSSED-LABEL", gh_long2scm(FL_EMBOSSED_LABEL)); 
  gh_define ("FL-BITMAP-LABEL", gh_long2scm(_FL_BITMAP_LABEL)); 
  gh_define ("FL-PIXMAP-LABEL", gh_long2scm(_FL_PIXMAP_LABEL)); 
  gh_define ("FL-IMAGE-LABEL", gh_long2scm(_FL_IMAGE_LABEL)); 
  gh_define ("FL-MULTI-LABEL", gh_long2scm(_FL_MULTI_LABEL)); 
  gh_define ("FL-FREE-LABELTYPE", gh_long2scm(FL_FREE_LABELTYPE)); 
   
  gh_define("FL-ALIGN-CENTER", gh_long2scm(FL_ALIGN_CENTER));
  gh_define("FL-ALIGN-TOP", gh_long2scm(FL_ALIGN_TOP));
  gh_define("FL-ALIGN-BOTTOM", gh_long2scm(FL_ALIGN_BOTTOM));
  gh_define("FL-ALIGN-LEFT", gh_long2scm(FL_ALIGN_LEFT));
  gh_define("FL-ALIGN-RIGHT", gh_long2scm(FL_ALIGN_RIGHT));
  gh_define("FL-ALIGN-INSIDE", gh_long2scm(FL_ALIGN_INSIDE));
  gh_define("FL-ALIGN-CLIP", gh_long2scm(FL_ALIGN_CLIP));
  gh_define("FL-ALIGN-WRAP", gh_long2scm(FL_ALIGN_WRAP));
  gh_define("FL-ALIGN-TOP-LEFT", gh_long2scm(FL_ALIGN_TOP_LEFT));
  gh_define("FL-ALIGN-TOP-RIGHT", gh_long2scm(FL_ALIGN_TOP_RIGHT));
  gh_define("FL-ALIGN-BOTTOM-LEFT", gh_long2scm(FL_ALIGN_BOTTOM_LEFT));
  gh_define("FL-ALIGN-BOTTOM-RIGHT", gh_long2scm(FL_ALIGN_BOTTOM_RIGHT));
  gh_define("FL-ALIGN-LEFT-TOP", gh_long2scm(FL_ALIGN_LEFT_TOP));
  gh_define("FL-ALIGN-RIGHT-TOP", gh_long2scm(FL_ALIGN_RIGHT_TOP));
  gh_define("FL-ALIGN-LEFT-BOTTOM", gh_long2scm(FL_ALIGN_LEFT_BOTTOM));
  gh_define("FL-ALIGN-RIGHT-BOTTOM", gh_long2scm(FL_ALIGN_RIGHT_BOTTOM));
  gh_define("FL-ALIGN-NOWRAP", gh_long2scm(FL_ALIGN_NOWRAP));
  
  gh_define("FL-HELVETICA", gh_long2scm(FL_HELVETICA));
  gh_define("FL-HELVETICA-BOLD", gh_long2scm(FL_HELVETICA_BOLD));
  gh_define("FL-HELVETICA-ITALIC", gh_long2scm(FL_HELVETICA_ITALIC));
  gh_define("FL-HELVETICA-BOLD-ITALIC", gh_long2scm(FL_HELVETICA_BOLD_ITALIC));
  gh_define("FL-COURIER", gh_long2scm(FL_COURIER));
  gh_define("FL-COURIER-BOLD", gh_long2scm(FL_COURIER_BOLD));
  gh_define("FL-COURIER-ITALIC", gh_long2scm(FL_COURIER_ITALIC));
  gh_define("FL-COURIER-BOLD-ITALIC", gh_long2scm(FL_COURIER_BOLD_ITALIC));
  gh_define("FL-TIMES", gh_long2scm(FL_TIMES));
  gh_define("FL-TIMES-BOLD", gh_long2scm(FL_TIMES_BOLD));
  gh_define("FL-TIMES-ITALIC", gh_long2scm(FL_TIMES_ITALIC));
  gh_define("FL-TIMES-BOLD-ITALIC", gh_long2scm(FL_TIMES_BOLD_ITALIC));
  gh_define("FL-SYMBOL", gh_long2scm(FL_SYMBOL));
  gh_define("FL-SCREEN", gh_long2scm(FL_SCREEN));
  gh_define("FL-SCREEN-BOLD", gh_long2scm(FL_SCREEN_BOLD));
  gh_define("FL-ZAPF-DINGBATS", gh_long2scm(FL_ZAPF_DINGBATS));
  gh_define("FL-FREE-FONT", gh_long2scm(FL_FREE_FONT));
  gh_define("FL-BOLD", gh_long2scm(FL_BOLD));
  gh_define("FL-ITALIC", gh_long2scm(FL_ITALIC));
  
  gh_define("FL-BLACK", gh_long2scm(FL_BLACK));
  gh_define("FL-RED", gh_long2scm(FL_RED));
  gh_define("FL-GREEN", gh_long2scm(FL_GREEN));
  gh_define("FL-YELLOW", gh_long2scm(FL_YELLOW));
  gh_define("FL-BLUE", gh_long2scm(FL_BLUE));
  gh_define("FL-MAGENTA", gh_long2scm(FL_MAGENTA));
  gh_define("FL-CYAN", gh_long2scm(FL_CYAN));
  gh_define("FL-WHITE", gh_long2scm(FL_WHITE));
  gh_define("FL-INACTIVE-COLOR", gh_long2scm(FL_INACTIVE_COLOR));
  gh_define("FL-SELECTION-COLOR", gh_long2scm(FL_SELECTION_COLOR));
  gh_define("FL-FREE-COLOR", gh_long2scm(FL_FREE_COLOR));
  gh_define("FL-NUM-FREE-COLOR", gh_long2scm(FL_NUM_FREE_COLOR));
  gh_define("FL-GRAY-RAMP", gh_long2scm(FL_GRAY_RAMP));
  gh_define("FL-GRAY0", gh_long2scm(FL_GRAY0));
  gh_define("FL-DARK3", gh_long2scm(FL_DARK3));
  gh_define("FL-DARK2", gh_long2scm(FL_DARK2));
  gh_define("FL-DARK1", gh_long2scm(FL_DARK1));
  gh_define("FL-GRAY", gh_long2scm(FL_GRAY));
  gh_define("FL-LIGHT1", gh_long2scm(FL_LIGHT1));
  gh_define("FL-LIGHT2", gh_long2scm(FL_LIGHT2));
  gh_define("FL-LIGHT3", gh_long2scm(FL_LIGHT3));
  gh_define("FL-COLOR-CUBE", gh_long2scm(FL_COLOR_CUBE));
  
  gh_define("FL-CURSOR-DEFAULT", gh_long2scm(FL_CURSOR_DEFAULT));
  gh_define("FL-CURSOR-ARROW", gh_long2scm(FL_CURSOR_ARROW));
  gh_define("FL-CURSOR-CROSS", gh_long2scm(FL_CURSOR_CROSS));
  gh_define("FL-CURSOR-WAIT", gh_long2scm(FL_CURSOR_WAIT));
  gh_define("FL-CURSOR-INSERT", gh_long2scm(FL_CURSOR_INSERT));
  gh_define("FL-CURSOR-HAND", gh_long2scm(FL_CURSOR_HAND));
  gh_define("FL-CURSOR-HELP", gh_long2scm(FL_CURSOR_HELP));
  gh_define("FL-CURSOR-MOVE", gh_long2scm(FL_CURSOR_MOVE));
  gh_define("FL-CURSOR-NS", gh_long2scm(FL_CURSOR_NS));
  gh_define("FL-CURSOR-WE", gh_long2scm(FL_CURSOR_WE));
  gh_define("FL-CURSOR-NWSE", gh_long2scm(FL_CURSOR_NWSE));
  gh_define("FL-CURSOR-NESW", gh_long2scm(FL_CURSOR_NESW));
  gh_define("FL-CURSOR-NONE", gh_long2scm(FL_CURSOR_NONE));
  gh_define("FL-CURSOR-N", gh_long2scm(FL_CURSOR_N));
  gh_define("FL-CURSOR-NE", gh_long2scm(FL_CURSOR_NE));
  gh_define("FL-CURSOR-E", gh_long2scm(FL_CURSOR_E));
  gh_define("FL-CURSOR-SE", gh_long2scm(FL_CURSOR_SE));
  gh_define("FL-CURSOR-S", gh_long2scm(FL_CURSOR_S));
  gh_define("FL-CURSOR-SW", gh_long2scm(FL_CURSOR_SW));
  gh_define("FL-CURSOR-W", gh_long2scm(FL_CURSOR_W));
  gh_define("FL-CURSOR-NW", gh_long2scm(FL_CURSOR_NW));
  
  gh_define("FL_READ", gh_long2scm(FL_READ));
  gh_define("FL_WRITE", gh_long2scm(FL_WRITE));
  gh_define("FL_EXCEPT", gh_long2scm(FL_EXCEPT));
  gh_define("FL_RGB", gh_long2scm(FL_RGB));
  gh_define("FL_INDEX", gh_long2scm(FL_INDEX));
  gh_define("FL_SINGLE", gh_long2scm(FL_SINGLE));
  gh_define("FL_DOUBLE", gh_long2scm(FL_DOUBLE));
  gh_define("FL_ACCUM", gh_long2scm(FL_ACCUM));
  gh_define("FL_ALPHA", gh_long2scm(FL_ALPHA));
  gh_define("FL_DEPTH", gh_long2scm(FL_DEPTH));
  gh_define("FL_STENCIL", gh_long2scm(FL_STENCIL));
  gh_define("FL_RGB8", gh_long2scm(FL_RGB8));
  gh_define("FL_MULTISAMPLE", gh_long2scm(FL_MULTISAMPLE));

}
