// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Group.H>
#include "guile_fltk.h"

long fltk_group_tag;

//***************************************************************************
//   
//  Fl_Group encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_group_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Group * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x, SCM_ARG1, "fltk-group-new");
  SCM_ASSERT (gh_number_p(y), y, SCM_ARG2, "fltk-group-new");
  SCM_ASSERT (gh_number_p(w), w, SCM_ARG3, "fltk-group-new");
  SCM_ASSERT (gh_number_p(h), h, SCM_ARG4, "fltk-group-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-group-new");

  widget = new Fl_Group(gh_scm2int(x), gh_scm2int(y), 
			gh_scm2int(w), gh_scm2int(h),
			gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_group_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_group_children (SCM widget_smob)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-children");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->children());
}

static SCM
fltk_group_child (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-child");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-group-child");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  return gfltk_lookup((Fl_Widget *) widget->child(gh_scm2int(a)));
}

// TODO:array() -- Needs to return a list of smobs of child widgets.

static SCM
fltk_group_add (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-add");
  SCM_ASSERT (gfltk_type_p(a, fltk_widget_tag),
              a, SCM_ARG2, "fltk-group-add");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->add((Fl_Widget *) gh_cdr (a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_add_resizable (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-add-resizable");
  SCM_ASSERT (gfltk_type_p(a, fltk_widget_tag),
              a, SCM_ARG2, "fltk-group-add-resizable");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->add_resizable(*(Fl_Widget *) gh_cdr (a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_begin (SCM widget_smob)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-begin");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->begin();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_end (SCM widget_smob)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-begin");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->end();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_set_current (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-set-current");
  SCM_ASSERT (gfltk_type_p(a, fltk_group_tag),
              a, SCM_ARG2, "fltk-group-set-current");

  widget = (Fl_Group *) gh_cdr (widget_smob);
  widget->current( (Fl_Group *) gh_cdr (a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_get_current (SCM widget_smob)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-get-current");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  return gfltk_lookup((Fl_Widget *) widget->current());
}

static SCM
fltk_group_find (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-find");
  SCM_ASSERT (gfltk_type_p(a, fltk_group_tag),
              a, SCM_ARG2, "fltk-group-find");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  return gh_int2scm(widget->find((Fl_Widget *) gh_cdr (a)));
}

static SCM
fltk_group_insert (SCM widget_smob, SCM a, SCM b)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-find");
  SCM_ASSERT (gfltk_type_p(a, fltk_widget_tag),
              a, SCM_ARG2, "fltk-group-find");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-group-find");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->insert(*(Fl_Widget *) gh_cdr (a), gh_scm2int(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_remove (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-remove");
  SCM_ASSERT (gfltk_type_p(a, fltk_group_tag),
              a, SCM_ARG2, "fltk-group-remove");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->remove((Fl_Widget *) gh_cdr (a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_clear (SCM widget_smob)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-clear");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->clear();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_set_resizable (SCM widget_smob, SCM a)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-set-resizable");
  SCM_ASSERT (gfltk_type_p(a, fltk_widget_tag),
              a, SCM_ARG2, "fltk-group-set-resizable");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  widget->resizable( (Fl_Widget *) gh_cdr (a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_group_get_resizable (SCM widget_smob)
{
  Fl_Group *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_group_tag),
              widget_smob, SCM_ARG1, "fltk-group-get-resizable");

  widget = (Fl_Group *) gh_cdr (widget_smob);

  return gfltk_lookup((Fl_Widget *) widget->resizable());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Group *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Group) + sizeof (Fl_Group));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Group *widget = (Fl_Group *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Group ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk_group scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_group_init ()
{
  fltk_group_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-group", fltk_group_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-group-new",
		       fltk_group_new);
  gh_new_procedure1_0 ("fltk-group-children",
		       fltk_group_children);
  gh_new_procedure2_0 ("fltk-group-child",
		       fltk_group_child);
  gh_new_procedure2_0 ("fltk-group-add",
		       fltk_group_add);
  gh_new_procedure2_0 ("fltk-group-add-resizable",
		       fltk_group_add_resizable);
  gh_new_procedure1_0 ("fltk-group-begin",
		       fltk_group_begin);
  gh_new_procedure1_0 ("fltk-group-end",
		       fltk_group_end);
  gh_new_procedure2_0 ("fltk-group-set-current",
		       fltk_group_set_current);
  gh_new_procedure1_0 ("fltk-group-get-current",
		       fltk_group_get_current);
  gh_new_procedure2_0 ("fltk-group-find",
		       fltk_group_find);
  gh_new_procedure3_0 ("fltk-group-insert",
		       fltk_group_insert);
  gh_new_procedure2_0 ("fltk-group-remove",
		       fltk_group_remove);
  gh_new_procedure1_0 ("fltk-group-clear",
		       fltk_group_clear);
  gh_new_procedure2_0 ("fltk-group-set-resizable",
		       fltk_group_set_resizable);
  gh_new_procedure1_0 ("fltk-group-get-resizable",
		       fltk_group_get_resizable);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_group_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_group_tag);
  fltk_widget_subclass(s, tag);

  sprintf(t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-group-new"));
  sprintf(t, "%s-children", s);
  gh_define (t, scm_symbol_value0("fltk-group-children"));
  sprintf(t, "%s-child", s);
  gh_define (t, scm_symbol_value0("fltk-group-child"));
  sprintf(t, "%s-add", s);
  gh_define (t, scm_symbol_value0("fltk-group-add"));
  sprintf(t, "%s-add-resizable", s);
  gh_define (t, scm_symbol_value0("fltk-group-add-resizable"));
  sprintf(t, "%s-begin", s);
  gh_define (t, scm_symbol_value0("fltk-group-begin"));
  sprintf(t, "%s-end", s);
  gh_define (t, scm_symbol_value0("fltk-group-end"));
  sprintf(t, "%s-set-current", s);
  gh_define (t, scm_symbol_value0("fltk-group-set-current"));
  sprintf(t, "%s-get-current", s);
  gh_define (t, scm_symbol_value0("fltk-group-get-current"));
  sprintf(t, "%s-find", s);
  gh_define (t, scm_symbol_value0("fltk-group-find"));
  sprintf(t, "%s-insert", s);
  gh_define (t, scm_symbol_value0("fltk-group-insert"));
  sprintf(t, "%s-remove", s);
  gh_define (t, scm_symbol_value0("fltk-group-remove"));
  sprintf(t, "%s-clear", s);
  gh_define (t, scm_symbol_value0("fltk-group-clear"));
  sprintf(t, "%s-set-resizable", s);
  gh_define (t, scm_symbol_value0("fltk-group-set-resizable"));
  sprintf(t, "%s-get-resizable", s);
  gh_define (t, scm_symbol_value0("fltk-group-get-resizable"));
}
