// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Input.H>
#include "guile_fltk.h"

long fltk_input_tag;

//***************************************************************************
//   
//  Fl_Input encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_input_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Input * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-input-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-input-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-input-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-input-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-input-new");

  widget = new Fl_Input(gh_scm2int(x), gh_scm2int(y), 
			gh_scm2int(w), gh_scm2int(h),
			gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_input_tag);
  gfltk_register(widget_smob);

  return widget_smob;
}

static SCM
fltk_input_set_cursor_color (SCM widget_smob, SCM a)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-set-cursor-color");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-set-cursor-color");
  
  widget = (Fl_Input *) gh_cdr (widget_smob);

  widget->cursor_color((Fl_Boxtype)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_input_get_cursor_color (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-get-cursor-color");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->cursor_color());
}

static SCM
fltk_input_index (SCM widget_smob, SCM a)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-index");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-index");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_char2scm(widget->index(gh_scm2int(a)));
}

static SCM
fltk_input_size (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-size");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_int2scm(widget->size());
}


static SCM
fltk_input_get_value (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-setonly");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return scm_makfrom0str(widget->value());
}

static SCM
fltk_input_set_value (SCM widget_smob, SCM a)
{
  Fl_Input *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-set-value");
  SCM_ASSERT (gh_string_p(a), a, SCM_ARG2, "fltk-input-set-value");
  
  widget = (Fl_Input *) gh_cdr (widget_smob);

  widget->value(gh_scm2newstr(a, NULL));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_input_set_when (SCM widget_smob, SCM a)
{
  Fl_Input *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-set-when");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-set-when");
  
  widget = (Fl_Input *) gh_cdr (widget_smob);

  widget->when(SCM_NFALSEP(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_input_get_when (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-get-when");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->when());
}

static SCM
fltk_input_set_textcolor (SCM widget_smob, SCM a)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-set-textcolor");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-set-textcolor");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  widget->textcolor(SCM_INUM(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_input_get_textcolor (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-get-textcolor");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textcolor());
}

static SCM
fltk_input_set_textfont (SCM widget_smob, SCM a)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-set-textfont");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-set-textfont");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  widget->textfont(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_input_get_textfont (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-get-textfont");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textfont());
}

static SCM
fltk_input_set_textsize (SCM widget_smob, SCM a)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-set-textsize");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-set-textsize");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  widget->textsize(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_input_get_textsize (SCM widget_smob)
{
  Fl_Input *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_tag),
              widget_smob, SCM_ARG1, "fltk-input-get-textsize");

  widget = (Fl_Input *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textsize());
}


//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Input *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Input) + sizeof (Fl_Input));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Input *widget = (Fl_Input *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Input ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk_box scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_input_init ()
{
  fltk_input_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_input_base_subclass("fltk-input", fltk_input_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-input-new",
		       fltk_input_new);
  gh_new_procedure2_0 ("fltk-input-set-cursor-color",
		       fltk_input_set_cursor_color);
  gh_new_procedure1_0 ("fltk-input-get-cursor-color",
		       fltk_input_get_cursor_color);
  gh_new_procedure2_0 ("fltk-input-index",
		       fltk_input_index);
  gh_new_procedure1_0 ("fltk-input-size",
		       fltk_input_size);
  gh_new_procedure1_0 ("fltk-input-get-value",
		       fltk_input_get_value);
  gh_new_procedure2_0 ("fltk-input-set-value",
		       fltk_input_set_value);
  gh_new_procedure2_0 ("fltk-input-set-when",
		       fltk_input_set_when);
  gh_new_procedure1_0 ("fltk-input-get-when",
		       fltk_input_get_when);
  gh_new_procedure2_0 ("fltk-input-set-textcolor",
		       fltk_input_set_textcolor);
  gh_new_procedure1_0 ("fltk-input-get-textcolor",
		       fltk_input_get_textcolor);
  gh_new_procedure2_0 ("fltk-input-set-textfont",
		       fltk_input_set_textfont);
  gh_new_procedure1_0 ("fltk-input-get-textfont",
		       fltk_input_get_textfont);
  gh_new_procedure2_0 ("fltk-input-set-textsize",
		       fltk_input_set_textsize);
  gh_new_procedure1_0 ("fltk-input-get-textsize",
		       fltk_input_get_textsize);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_input_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_input_tag);
  fltk_input_base_subclass(s, tag);

  sprintf(t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-input-new"));
  sprintf(t, "%s-set-cursor-color", s);
  gh_define (t, scm_symbol_value0("fltk-input-set-cursor-color"));
  sprintf(t, "%s-get-cursor-color", s);
  gh_define (t, scm_symbol_value0("fltk-input-get-cursor-color"));
  sprintf(t, "%s-index", s);
  gh_define (t, scm_symbol_value0("fltk-input-index"));
  sprintf(t, "%s-size", s);
  gh_define (t, scm_symbol_value0("fltk-input-size"));
  sprintf(t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-input-get-value"));
  sprintf(t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-input-set-value"));
  sprintf(t, "%s-set-when", s);
  gh_define (t, scm_symbol_value0("fltk-input-set-when"));
  sprintf(t, "%s-get-when", s);
  gh_define (t, scm_symbol_value0("fltk-input-get-when"));
  sprintf(t, "%s-set-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-input-set-textcolor"));
  sprintf(t, "%s-get-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-input-get-textcolor"));
  sprintf(t, "%s-set-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-input-set-textfont"));
  sprintf(t, "%s-get-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-input-get-textfont"));
  sprintf(t, "%s-set-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-input-set-textsize"));
  sprintf(t, "%s-get-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-input-get-textsize"));
}
