// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Input_.H>
#include "guile_fltk.h"

long fltk_input_base_tag;

//***************************************************************************
//   
//  Fl_Input_ encapsulation method(s):
//
//***************************************************************************

/*
static SCM
fltk_input_base_wordboundary(SCM widget_smob, SCM a)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-wordboundary");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  if (gh_int2scm((uchar)widget->wordboundary(aa)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_lineboundary(SCM widget_smob, SCM a)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-lineboundary");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  if (gh_int2scm((uchar)widget->lineboundary(aa)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_drawtext(SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-drawtext");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  int bb = gh_scm2int(b);
  int cc = gh_scm2int(c);
  int dd = gh_scm2int(d);

  if (gh_int2scm((uchar)widget->drawtext(aa, bb, cc, dd)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_handletext(SCM widget_smob, SCM a, SCM b, SCM c, SCM d, SCM e)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-handletext");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  int bb = gh_scm2int(b);
  int cc = gh_scm2int(c);
  int dd = gh_scm2int(d);
  int ee = gh_scm2int(e);

  if (gh_int2scm((uchar)widget->handletext(aa, bb, cc, dd, ee)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_up_down_position(SCM widget_smob, SCM a, SCM b)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-up-down-position");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  int bb = gh_scm2int(b);

  if (gh_int2scm((uchar)widget->up_down_position(aa, bb)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_maybe_do_callback(SCM widget_smob)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-maybe-do-callback");

  //TODO: ASSERT for double argument
  widget = (Fl_Input_ *) gh_cdr (widget_smob);
  widget->maybe_do_callback();

  return SCM_UNSPECIFIED;
}
*/

static SCM
fltk_input_base_get_position(SCM widget_smob)
{
  Fl_Input_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-get-position");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->position());
}

static SCM
fltk_input_base_set_position(SCM widget_smob, SCM a, SCM b)
{
  Fl_Input_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-set-position");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-input-base-set-position");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-input-base-set-position");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->position(gh_scm2int(a), gh_scm2int(b)));
}

static SCM
fltk_input_base_replace(SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Input_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-replace");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-input-base-replace");
  SCM_ASSERT (gh_number_p(b), b, SCM_ARG3, "fltk-input-base-replace");
  SCM_ASSERT (gh_string_p(c), c, SCM_ARG4, "fltk-input-base-replace");
  SCM_ASSERT (gh_number_p(d), d, SCM_ARG5, "fltk-input-base-replace");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->replace(gh_scm2int(a), gh_scm2int(b), 
				    gh_scm2newstr(c, NULL), gh_scm2int(d)));
}

static SCM
fltk_input_base_copy(SCM widget_smob)
{
  Fl_Input_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-copy");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->copy());
}

static SCM
fltk_input_base_undo(SCM widget_smob)
{
  Fl_Input_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-undo");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->undo());
}

static SCM
fltk_input_base_copy_cuts(SCM widget_smob)
{
  Fl_Input_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base-copy-cuts");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  return gh_int2scm(widget->copy_cuts());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Input_ *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Input_) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Input_ *widget = (Fl_Input_ *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Input_ ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-value_input scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_input_base_init ()
{
  fltk_input_base_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-value-input", fltk_input_base_tag);

  // add new methods
  //gh_new_procedure1_0 ("fltk-input-base-wordboundary", 
  //		       fltk_input_base_wordboundary);
  //  gh_new_procedure1_0 ("fltk-input-base-lineboundary", 
  //		       fltk_input_base_lineboundary);
  //  gh_new_procedure5_0 ("fltk-input-base-drawtext", 
  //		       fltk_input_base_drawtext);
  //  gh_new_procedure6_0 ("fltk-input-base-handletext", 
  //		       fltk_input_base_handletext);
  //  gh_new_procedure3_0 ("fltk-input-base-up-down-position", 
  //		       fltk_input_base_up_down_position);
  //  gh_new_procedure1_0 ("fltk-input-base-maybe-do-callback", 
  //		       fltk_input_base_maybe_do_callback);
  gh_new_procedure1_0 ("fltk-input-base-get-position", 
		       fltk_input_base_get_position);
  gh_new_procedure3_0 ("fltk-input-base-set-position", 
		       fltk_input_base_set_position);
  gh_new_procedure5_0 ("fltk-input-base-replace", 
		       fltk_input_base_replace);
  gh_new_procedure1_0 ("fltk-input-base-copy", 
		       fltk_input_base_copy);
  gh_new_procedure1_0 ("fltk-input-base-undo", 
		       fltk_input_base_undo);
  gh_new_procedure1_0 ("fltk-input-base-copy-cuts", 
		       fltk_input_base_copy_cuts);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_input_base_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_input_base_tag);
  fltk_widget_subclass(s, tag);

  /*  
      sprintf (t, "%s-wordboundary", s);
      gh_define (t, scm_symbol_value0("fltk-input-base-wordboundary"));
      sprintf (t, "%s-lineboundary", s);
      gh_define (t, scm_symbol_value0("fltk-input-base-lineboundary"));
      sprintf (t, "%s-drawtext", s);
      gh_define (t, scm_symbol_value0("fltk-input-base-drawtext"));
      sprintf (t, "%s-handletext", s);
      gh_define (t, scm_symbol_value0("fltk-input-base-handletext"));
      sprintf (t, "%s-up-down-position", s);
      gh_define (t, scm_symbol_value0("fltk-input-base-up-down-position"));
      sprintf (t, "%s-maybe-do-callback", s);
      gh_define (t, scm_symbol_value0("fltk-input-base-maybe-do-callback"));
  */

  sprintf (t, "%s-get-position", s);
  gh_define (t, scm_symbol_value0("fltk-input-base-get-position"));
  sprintf (t, "%s-set-position", s);
  gh_define (t, scm_symbol_value0("fltk-input-base-set-position"));
  sprintf (t, "%s-replace", s);
  gh_define (t, scm_symbol_value0("fltk-input-base-replace"));
  sprintf (t, "%s-copy", s);
  gh_define (t, scm_symbol_value0("fltk-input-base-copy"));
  sprintf (t, "%s-undo", s);
  gh_define (t, scm_symbol_value0("fltk-input-base-undo"));
  sprintf (t, "%s-copy-cuts", s);
  gh_define (t, scm_symbol_value0("fltk-input-base-copy-cuts"));

}
