// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Menu_.H>
#include "guile_fltk.h"

long fltk_menu_tag;

//***************************************************************************
//   
//  Fl_Menu_ encapsulation method(s):
//
//***************************************************************************

static SCM
fltk_menu_set_value (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-set-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-set-value");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->value(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_get_value (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-value");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->value());
}

//TODO:test_shortcut is missing

static SCM
fltk_menu_global (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-value");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);
  
  widget->global();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_text (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-text");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-text");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return scm_makfrom0str (widget->text(gh_scm2int(a)));
}

static SCM
fltk_menu_size (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-value");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->size());
}

static SCM
fltk_menu_add (SCM widget_smob, SCM title, SCM shortcut, SCM callback, SCM flags)
{
  Fl_Menu_ *widget;
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-add");
  SCM_ASSERT (gh_string_p(title), title, SCM_ARG2, "fltk-menu-add");
  SCM_ASSERT (gh_string_p(shortcut), shortcut, SCM_ARG3, "fltk-menu-add");
  SCM_ASSERT ((gh_procedure_p(callback) || gh_boolean_p(callback)), callback, SCM_ARG4, "fltk-menu-add");
  SCM_ASSERT (gh_number_p(flags), flags,  SCM_ARG5, "fltk-menu-add");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  if (gh_boolean_p(callback))
    widget->add(gh_scm2newstr(title, NULL), 
		gh_scm2newstr(shortcut, NULL), 
		0, 0, gh_scm2int(flags));
  else
    widget->add(gh_scm2newstr(title, NULL), 
		gh_scm2newstr(shortcut, NULL), 
		guile_fltk_callback2, 
		(void *)callback, gh_scm2int(flags));
  
  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_clear (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-clear");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->clear();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_replace (SCM widget_smob, SCM a, SCM b)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-replace");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-menu-replace");
  SCM_ASSERT (gh_string_p(b), b, SCM_ARG3, "fltk-menu-replace");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->replace(gh_scm2int(a), gh_scm2newstr(b, NULL));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_remove (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-remove");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-remove");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->remove(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_shortcut (SCM widget_smob, SCM a, SCM b)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-shortcut");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-shortcut");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-menu-shortcut");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->shortcut(gh_scm2int(a), gh_scm2int(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_mode (SCM widget_smob, SCM a, SCM b)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-mode");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-mode");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-menu-mode");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->mode(gh_scm2int(a), gh_scm2int(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_set_textcolor (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-set-textcolor");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-set-textcolor");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->textcolor(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_get_textcolor (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-textcolor");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textcolor());
}

static SCM
fltk_menu_set_textfont (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-set-textfont");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-set-textfont");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->textfont(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_get_textfont (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-textfont");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textfont());
}

static SCM
fltk_menu_set_textsize (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-set-textsize");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-set-textsize");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->textsize(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_get_textsize (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-textsize");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textsize());
}

static SCM
fltk_menu_set_down_box (SCM widget_smob, SCM a)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-set-down-box");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-menu-set-down-box");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  widget->down_box((Fl_Boxtype)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_menu_get_down_box (SCM widget_smob)
{
  Fl_Menu_ *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_menu_tag),
              widget_smob, SCM_ARG1, "fltk-menu-get-down-box");

  widget = (Fl_Menu_ *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->down_box());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Menu_ *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Gfltk) + sizeof (Fl_Menu_));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Menu_ *widget = (Fl_Menu_ *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Menu_ ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-menu scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_menu_init ()
{
  fltk_menu_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-menu", fltk_menu_tag);

  // add new methods
  gh_new_procedure2_0 ("fltk-menu-set-value",
		       fltk_menu_set_value);
  gh_new_procedure1_0 ("fltk-menu-get-value",
		       fltk_menu_get_value);
  gh_new_procedure1_0 ("fltk-menu-global",
		       fltk_menu_global);
  gh_new_procedure2_0 ("fltk-menu-text",
		       fltk_menu_text);
  gh_new_procedure1_0 ("fltk-menu-size",
		       fltk_menu_size);
  gh_new_procedure5_0 ("fltk-menu-add",
		       fltk_menu_add);
  gh_new_procedure1_0 ("fltk-menu-clear",
		       fltk_menu_clear);
  gh_new_procedure3_0 ("fltk-menu-replace",
		       fltk_menu_replace);
  gh_new_procedure2_0 ("fltk-menu-remove",
		       fltk_menu_remove);
  gh_new_procedure3_0 ("fltk-menu-shortcut",
		       fltk_menu_shortcut);
  gh_new_procedure3_0 ("fltk-menu-mode",
		       fltk_menu_mode);
  gh_new_procedure2_0 ("fltk-menu-set-textcolor",
		       fltk_menu_set_textcolor);
  gh_new_procedure1_0 ("fltk-menu-get-textcolor",
		       fltk_menu_get_textcolor);
  gh_new_procedure2_0 ("fltk-menu-set-textfont",
		       fltk_menu_set_textfont);
  gh_new_procedure1_0 ("fltk-menu-get-textfont",
		       fltk_menu_get_textfont);
  gh_new_procedure2_0 ("fltk-menu-set-textsize",
		       fltk_menu_set_textsize);
  gh_new_procedure1_0 ("fltk-menu-get-textsize",
		       fltk_menu_get_textsize);
  gh_new_procedure2_0 ("fltk-menu-set-down-box",
		       fltk_menu_set_down_box);
  gh_new_procedure1_0 ("fltk-menu-get-down-box",
		       fltk_menu_get_down_box);

  gh_define ("FL_MENU_INACTIVE", gh_int2scm(1));
  gh_define ("FL_MENU_TOGGLE", gh_int2scm(2));
  gh_define ("FL_MENU_VALUE", gh_int2scm(4));
  gh_define ("FL_MENU_RADIO", gh_int2scm(8));
  gh_define ("FL_MENU_INVISIBLE", gh_int2scm(16));
  gh_define ("FL_SUBMENU_POINTER", gh_int2scm(32));
  gh_define ("FL_SUBMENU", gh_int2scm(64));
  gh_define ("FL_MENU_DIVIDER", gh_int2scm(128));
  gh_define ("FL_MENU_HORIZONTAL", gh_int2scm(256));
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_menu_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_menu_tag);
  fltk_widget_subclass(s, tag);

  sprintf(t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-menu-set-value"));
  sprintf(t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-menu-get-value"));
  sprintf(t, "%s-global", s);
  gh_define (t, scm_symbol_value0("fltk-menu-global"));
  sprintf(t, "%s-text", s);
  gh_define (t, scm_symbol_value0("fltk-menu-text"));
  sprintf(t, "%s-size", s);
  gh_define (t, scm_symbol_value0("fltk-menu-size"));
  sprintf(t, "%s-add", s);
  gh_define (t, scm_symbol_value0("fltk-menu-add"));
  sprintf(t, "%s-clear", s);
  gh_define (t, scm_symbol_value0("fltk-menu-clear"));
  sprintf(t, "%s-replace", s);
  gh_define (t, scm_symbol_value0("fltk-menu-replace"));
  sprintf(t, "%s-remove", s);
  gh_define (t, scm_symbol_value0("fltk-menu-remove"));
  sprintf(t, "%s-shurtcut", s);
  gh_define (t, scm_symbol_value0("fltk-menu-shortcut"));
  sprintf(t, "%s-mode", s);
  gh_define (t, scm_symbol_value0("fltk-menu-mode"));
  sprintf(t, "%s-set-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-menu-set-textcolor"));
  sprintf(t, "%s-get-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-menu-get-textcolor"));
  sprintf(t, "%s-set-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-menu-set-textfont"));
  sprintf(t, "%s-get-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-menu-get-textfont"));
  sprintf(t, "%s-set-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-menu-set-textsize"));
  sprintf(t, "%s-get-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-menu-get-textsize"));
  sprintf(t, "%s-set-down-box", s);
  gh_define (t, scm_symbol_value0("fltk-menu-set-down-box"));
  sprintf(t, "%s-get-down-box", s);
  gh_define (t, scm_symbol_value0("fltk-menu-get-down-box"));
}
