// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Multi_Browser.H>
#include "guile_fltk.h"

long fltk_multi_browser_tag;

//***************************************************************************
//   
//  Fl_Browser encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_multi_browser_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Multi_Browser * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-multi-browser-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-multi-browser-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-multi-browser-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-multi-browser-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-multi-browser-new");

  widget = new Fl_Multi_Browser(gh_scm2int(x), gh_scm2int(y), 
				gh_scm2int(w), gh_scm2int(h),
				gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_multi_browser_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_multi_browser_deselect (SCM widget_smob)
{
  Fl_Multi_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_multi_browser_tag),
              widget_smob, SCM_ARG1, "fltk-multi-browser-add");

  widget = (Fl_Multi_Browser *) gh_cdr (widget_smob);

  if (widget->deselect())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

static SCM
fltk_multi_browser_select (SCM widget_smob, SCM a, SCM b)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-multi-browser-select");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-multi-browser-select");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-multi-browser-select");

  widget = (Fl_Multi_Browser *) gh_cdr (widget_smob);

  widget->select(gh_scm2int(a), gh_scm2int(b));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_multi_browser_selected (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-multi-browser-select");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-multi-browser-select");

  widget = (Fl_Multi_Browser *) gh_cdr (widget_smob);

  if (widget->selected(gh_scm2int(a)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

static SCM
fltk_multi_browser_set_value (SCM widget_smob, SCM a)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-multi-browser-set-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-multi-browser-set-value");

  widget = (Fl_Multi_Browser *) gh_cdr (widget_smob);

  widget->value(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_multi_browser_get_value (SCM widget_smob)
{
  Fl_Browser *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_browser_tag),
              widget_smob, SCM_ARG1, "fltk-multi-browser-get-value");

  widget = (Fl_Multi_Browser *) gh_cdr (widget_smob);

  return SCM_MAKINUM(widget->value());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Multi_Browser *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Multi_Browser) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Multi_Browser *widget = (Fl_Multi_Browser *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Multi_Browser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-multi-browser scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_multi_browser_init ()
{
  fltk_multi_browser_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_browser_subclass("fltk-multi-browser", fltk_browser_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-multi-browser-new", 
		       fltk_multi_browser_new);
  gh_new_procedure1_0 ("fltk-multi-browser-deselect", 
		       fltk_multi_browser_deselect);
  gh_new_procedure3_0 ("fltk-multi-browser-select", 
		       fltk_multi_browser_select);
  gh_new_procedure2_0 ("fltk-multi-browser-selected", 
		       fltk_multi_browser_selected);
  gh_new_procedure2_0 ("fltk-multi-browser-set-value", 
		       fltk_multi_browser_set_value);
  gh_new_procedure1_0 ("fltk-multi-browser-get-value", 
		       fltk_multi_browser_get_value);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_multi_browser_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_multi_browser_tag);
  fltk_browser_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-browser-new"));
  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-multi-browser-new"));
  sprintf (t, "%s-deselect", s);
  gh_define (t, scm_symbol_value0("fltk-multi-browser-deselect"));
  sprintf (t, "%s-select", s);
  gh_define (t, scm_symbol_value0("fltk-multi-browser-select"));
  sprintf (t, "%s-selected", s);
  gh_define (t, scm_symbol_value0("fltk-multi-browser-selected"));
  sprintf (t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-multi-browser-set-value"));
  sprintf (t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-multi-browser-get-value"));
}
