// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Output.H>
#include "guile_fltk.h"

long fltk_output_tag;

//***************************************************************************
//   
//  Fl_Output encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_output_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Output * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-output-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-output-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-output-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-output-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-output-new");

  widget = new Fl_Output(gh_scm2int(x), gh_scm2int(y), 
			  gh_scm2int(w), gh_scm2int(h),
			  gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_output_tag);
  gfltk_register(widget_smob);

  return widget_smob;
}

static SCM
fltk_output_set_cursor_color (SCM widget_smob, SCM a)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-set-cursor-color");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-output-set-cursor-color");
  
  widget = (Fl_Output *) gh_cdr (widget_smob);

  widget->cursor_color((Fl_Boxtype)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_output_get_cursor_color (SCM widget_smob)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-get-cursor-color");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->cursor_color());
}

static SCM
fltk_output_index (SCM widget_smob, SCM a)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-index");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-output-index");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_char2scm(widget->index(gh_scm2int(a)));
}

static SCM
fltk_output_size (SCM widget_smob)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-size");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_int2scm(widget->size());
}


static SCM
fltk_output_get_value (SCM widget_smob)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-get-value");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_str02scm((char *)widget->value());
}

static SCM
fltk_output_set_value (SCM widget_smob, SCM a)
{
  Fl_Output *widget;
  
  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-set-value");
  SCM_ASSERT (gh_string_p(a), a, SCM_ARG2, "fltk-output-set-value");
  
  widget = (Fl_Output *) gh_cdr (widget_smob);

  widget->value(SCM_CHARS(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_output_set_textcolor (SCM widget_smob, SCM a)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-set-textcolor");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-output-set-textcolor");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  widget->textcolor(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_output_get_textcolor (SCM widget_smob)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-get-textcolor");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textcolor());
}

static SCM
fltk_output_set_textfont (SCM widget_smob, SCM a)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-set-textfont");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-output-set-textfont");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  widget->textfont(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_output_get_textfont (SCM widget_smob)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-get-textfont");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textfont());
}

static SCM
fltk_output_set_textsize (SCM widget_smob, SCM a)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-set-textsize");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-output-set-textsize");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  widget->textsize(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_output_get_textsize (SCM widget_smob)
{
  Fl_Output *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_output_tag),
              widget_smob, SCM_ARG1, "fltk-output-get-textsize");

  widget = (Fl_Output *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->textsize());
}


//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Output *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Output) + sizeof (Fl_Output));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Output *widget = (Fl_Output *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Output ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk_box scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_output_init ()
{
  fltk_output_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_input_base_subclass("fltk-output", fltk_output_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-output-new",
		       fltk_output_new);
  gh_new_procedure2_0 ("fltk-output-set-cursor-color",
		       fltk_output_set_cursor_color);
  gh_new_procedure1_0 ("fltk-output-get-cursor-color",
		       fltk_output_get_cursor_color);
  gh_new_procedure2_0 ("fltk-output-index",
		       fltk_output_index);
  gh_new_procedure1_0 ("fltk-output-size",
		       fltk_output_size);
  gh_new_procedure1_0 ("fltk-output-get-value",
		       fltk_output_get_value);
  gh_new_procedure2_0 ("fltk-output-set-value",
		       fltk_output_set_value);
  gh_new_procedure2_0 ("fltk-output-set-textcolor",
		       fltk_output_set_textcolor);
  gh_new_procedure1_0 ("fltk-output-get-textcolor",
		       fltk_output_get_textcolor);
  gh_new_procedure2_0 ("fltk-output-set-textfont",
		       fltk_output_set_textfont);
  gh_new_procedure1_0 ("fltk-output-get-textfont",
		       fltk_output_get_textfont);
  gh_new_procedure2_0 ("fltk-output-set-textsize",
		       fltk_output_set_textsize);
  gh_new_procedure1_0 ("fltk-output-get-textsize",
		       fltk_output_get_textsize);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_output_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_output_tag);
  fltk_input_base_subclass(s, tag);

  sprintf(t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-output-new"));
  sprintf(t, "%s-set-cursor-color", s);
  gh_define (t, scm_symbol_value0("fltk-output-set-cursor-color"));
  sprintf(t, "%s-get-cursor-color", s);
  gh_define (t, scm_symbol_value0("fltk-output-get-cursor-color"));
  sprintf(t, "%s-index", s);
  gh_define (t, scm_symbol_value0("fltk-output-index"));
  sprintf(t, "%s-size", s);
  gh_define (t, scm_symbol_value0("fltk-output-size"));
  sprintf(t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-output-get-value"));
  sprintf(t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-output-set-value"));
  sprintf(t, "%s-set-when", s);
  gh_define (t, scm_symbol_value0("fltk-output-set-when"));
  sprintf(t, "%s-get-when", s);
  gh_define (t, scm_symbol_value0("fltk-output-get-when"));
  sprintf(t, "%s-set-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-output-set-textcolor"));
  sprintf(t, "%s-get-textcolor", s);
  gh_define (t, scm_symbol_value0("fltk-output-get-textcolor"));
  sprintf(t, "%s-set-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-output-set-textfont"));
  sprintf(t, "%s-get-textfont", s);
  gh_define (t, scm_symbol_value0("fltk-output-get-textfont"));
  sprintf(t, "%s-set-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-output-set-textsize"));
  sprintf(t, "%s-get-textsize", s);
  gh_define (t, scm_symbol_value0("fltk-output-get-textsize"));
}
