// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Roller.H>
#include "guile_fltk.h"

long fltk_roller_tag;

//***************************************************************************
//   
//  Fl_Roller encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_roller_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Roller * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-roller-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-roller-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-roller-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-roller-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-roller-new");

  widget = new Fl_Roller(gh_scm2int(x), gh_scm2int(y), 
			  gh_scm2int(w), gh_scm2int(h),
			  gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_roller_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Roller *) SCM_CDR (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Roller) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Roller *widget = (Fl_Roller *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Roller ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-roller scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_roller_init ()
{
  fltk_roller_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_valuator_subclass("fltk-roller", fltk_roller_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-roller-new", 
		       fltk_roller_new);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_roller_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_roller_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-roller-new"));

}
