// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Scrollbar.H>
#include "guile_fltk.h"

long fltk_scrollbar_tag;

//***************************************************************************
//   
//  Fl_Scrollbar encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_scrollbar_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Scrollbar * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-scrollbar-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-scrollbar-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-scrollbar-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-scrollbar-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-scrollbar-new");

  widget = new Fl_Scrollbar(gh_scm2int(x), gh_scm2int(y), 
			  gh_scm2int(w), gh_scm2int(h),
			  gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_scrollbar_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_scrollbar_get_linesize(SCM widget_smob)
{
  Fl_Scrollbar *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_scrollbar_tag),
              widget_smob, SCM_ARG1, "fltk-scrollbar-get-linesize");

  widget = (Fl_Scrollbar *) gh_cdr (widget_smob);
  return gh_int2scm(widget->linesize());
}

static SCM
fltk_scrollbar_set_linesize(SCM widget_smob, SCM a)
{
  Fl_Scrollbar *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_scrollbar_tag),
              widget_smob, SCM_ARG1, "fltk-scrollbar-set-linesize");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-new");

  widget = (Fl_Scrollbar *) gh_cdr (widget_smob);

  widget->linesize(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_scrollbar_get_value(SCM widget_smob)
{
  Fl_Scrollbar *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_scrollbar_tag),
              widget_smob, SCM_ARG1, "fltk-scrollbar-get-value");

  widget = (Fl_Scrollbar *) gh_cdr (widget_smob);
  return gh_int2scm(widget->value());
}

static SCM
fltk_scrollbar_set_value(SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Scrollbar *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_scrollbar_tag),
              widget_smob, SCM_ARG1, "fltk-scrollbar-set-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-new");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-browser-new");
  SCM_ASSERT (gh_number_p(c), c,  SCM_ARG4, "fltk-browser-new");
  SCM_ASSERT (gh_number_p(d), d,  SCM_ARG5, "fltk-browser-new");

  widget = (Fl_Scrollbar *) gh_cdr (widget_smob);

  gh_int2scm(widget->value(gh_scm2int(a), gh_scm2int(b), 
    gh_scm2int(c), gh_scm2int(d)));

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Scrollbar *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Scrollbar) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Scrollbar *widget = (Fl_Scrollbar *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Scrollbar ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-scrollbar scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_scrollbar_init ()
{
  fltk_scrollbar_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_slider_subclass("fltk-scrollbar", fltk_scrollbar_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-scrollbar-new", 
		       fltk_scrollbar_new);
  gh_new_procedure1_0 ("fltk-scrollbar-get-linesize", 
		       fltk_scrollbar_get_linesize);
  gh_new_procedure2_0 ("fltk-scrollbar-set-linesize", 
		       fltk_scrollbar_set_linesize);
  gh_new_procedure1_0 ("fltk-scrollbar-get-value", 
		       fltk_scrollbar_get_value);
  gh_new_procedure5_0 ("fltk-scrollbar-set-value", 
		       fltk_scrollbar_set_value);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_scrollbar_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_scrollbar_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-scrollbar-new"));
  sprintf (t, "%s-get-linesize", s);
  gh_define (t, scm_symbol_value0("fltk-scrollbar-get-linesize"));
  sprintf (t, "%s-set-linesize", s);
  gh_define (t, scm_symbol_value0("fltk-scrollbar-set-linesize"));
  sprintf (t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-scrollbar-get-value"));
  sprintf (t, "%s-set-value", s);
  gh_define (t, scm_symbol_value0("fltk-scrollbar-set-value"));
}
