// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Secret_Input.H>
#include "guile_fltk.h"

long fltk_secret_input_tag;

//***************************************************************************
//   
//  Fl_Secret_Input encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_secret_input_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Secret_Input * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x, SCM_ARG1, "fltk-secret-input-new");
  SCM_ASSERT (gh_number_p(y), y, SCM_ARG2, "fltk-secret-input-new");
  SCM_ASSERT (gh_number_p(w), w, SCM_ARG3, "fltk-secret-input-new");
  SCM_ASSERT (gh_number_p(h), h, SCM_ARG4, "fltk-secret-input-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-secret-input-new");

  widget = new Fl_Secret_Input(gh_scm2int(x), gh_scm2int(y), 
			  gh_scm2int(w), gh_scm2int(h),
			  gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_secret_input_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Secret_Input *) SCM_CDR (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Secret_Input) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Secret_Input *widget = (Fl_Secret_Input *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Secret_Input ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-secret-input scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_secret_input_init ()
{
  fltk_secret_input_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-secret-input", fltk_secret_input_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-secret-input-new", 
		       fltk_secret_input_new);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_secret_input_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_secret_input_tag);
  fltk_input_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-secret-input-new"));
}
