// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Slider.H>
#include "guile_fltk.h"

long fltk_slider_tag;

//***************************************************************************
//   
//  Fl_Slider encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_slider_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Slider * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-slider-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-slider-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-slider-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-slider-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-slider-new");

  widget = new Fl_Slider(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_slider_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_slider_set_type(SCM widget_smob, SCM a)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-set-type");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-slider-set-type");

  widget = (Fl_Slider *) gh_cdr (widget_smob);
  widget->type((uchar)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_slider_get_type(SCM widget_smob)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-get-type");

  widget = (Fl_Slider *) gh_cdr (widget_smob);

  return gh_int2scm(widget->type());
}


static SCM
fltk_slider_set_slider(SCM widget_smob, SCM a)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-set-slider");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-browser-new");

  widget = (Fl_Slider *) gh_cdr (widget_smob);

  widget->slider((Fl_Boxtype)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_slider_get_slider(SCM widget_smob)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-get-slider");

  widget = (Fl_Slider *) gh_cdr (widget_smob);

  return gh_int2scm(widget->slider());
}


static SCM
fltk_slider_set_slider_size(SCM widget_smob, SCM a)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-set-type");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-slider-set-type");

  widget = (Fl_Slider *) gh_cdr (widget_smob);
  widget->slider_size(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_slider_get_slider_size(SCM widget_smob)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-get-type");

  widget = (Fl_Slider *) gh_cdr (widget_smob);

  return gh_double2scm(widget->slider_size());
}

static SCM
fltk_slider_scrollvalue(SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Slider *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_slider_tag),
              widget_smob, SCM_ARG1, "fltk-slider-scrollvalue");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-slider-scrollvalue");
  SCM_ASSERT (gh_number_p(b), b, SCM_ARG3, "fltk-slider-scrollvalue");
  SCM_ASSERT (gh_number_p(c), c, SCM_ARG4, "fltk-slider-scrollvalue");
  SCM_ASSERT (gh_number_p(d), d, SCM_ARG5, "fltk-slider-scrollvalue");

  widget = (Fl_Slider *) gh_cdr (widget_smob);

  return gh_int2scm(widget->scrollvalue(gh_scm2int(a), 
					gh_scm2int(b), 
					gh_scm2int(c), 
					gh_scm2int(d)));
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Slider *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Slider) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Slider *widget = (Fl_Slider *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Slider ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-slider scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_slider_init ()
{
  fltk_slider_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_valuator_subclass("fltk-slider", fltk_slider_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-slider-new", 
		       fltk_slider_new);
  gh_new_procedure1_0 ("fltk-slider-get-type", 
		       fltk_slider_get_type);
  gh_new_procedure2_0 ("fltk-slider-set-type", 
		       fltk_slider_set_type);
  gh_new_procedure2_0 ("fltk-slider-set-slider", 
		       fltk_slider_set_slider);
  gh_new_procedure1_0 ("fltk-slider-get-slider", 
		       fltk_slider_get_slider);
  gh_new_procedure2_0 ("fltk-slider-set-slider-size", 
		       fltk_slider_set_slider_size);
  gh_new_procedure1_0 ("fltk-slider-get-slider-size", 
		       fltk_slider_get_slider_size);
  gh_new_procedure5_0 ("fltk-slider-scrollvalue", 
		       fltk_slider_scrollvalue);
  gh_new_procedure2_0 ("fltk-slider-set-type", 
		       fltk_slider_set_type);
  gh_new_procedure1_0 ("fltk-slider-get-type", 
		       fltk_slider_get_type);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_slider_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_slider_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-slider-new"));
  sprintf (t, "%s-get-type", s);
  gh_define (t, scm_symbol_value0("fltk-slider-get-type"));
  sprintf (t, "%s-set-type", s);
  gh_define (t, scm_symbol_value0("fltk-slider-set-type"));
  sprintf (t, "%s-set-slider", s);
  gh_define (t, scm_symbol_value0("fltk-slider-set-slider"));
  sprintf (t, "%s-get-slider", s);
  gh_define (t, scm_symbol_value0("fltk-slider-get-slider"));
  sprintf (t, "%s-set-slider-size", s);
  gh_define (t, scm_symbol_value0("fltk-slider-set-slider-size"));
  sprintf (t, "%s-get-slider-size", s);
  gh_define (t, scm_symbol_value0("fltk-slider-get-slider-size"));
  sprintf (t, "%s-scrollvalue", s);
  gh_define (t, scm_symbol_value0("fltk-slider-scrollvalue"));
}
