// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Tile.H>
#include "guile_fltk.h"

long fltk_tile_tag;

//***************************************************************************
//   
//  Fl_Tile encapsulation method(s):
//
//***************************************************************************


static SCM 
fltk_tile_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Tile * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-tile-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-tile-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-tile-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-tile-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-tile-new");

  widget = new Fl_Tile(gh_scm2int(x), gh_scm2int(y), 
		       gh_scm2int(w), gh_scm2int(h),
		       gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_tile_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}


static SCM
fltk_tile_position (SCM widget_smob, SCM sx, SCM sy, SCM tx, SCM ty)
{
  Fl_Tile *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_tile_tag),
              widget_smob, SCM_ARG1, "fltk-tile-position");
  SCM_ASSERT (gh_number_p(sx), sx,  SCM_ARG2, "fltk-tile-position");
  SCM_ASSERT (gh_number_p(sy), sy,  SCM_ARG3, "fltk-tile-position");
  SCM_ASSERT (gh_number_p(tx), tx,  SCM_ARG4, "fltk-tile-position");
  SCM_ASSERT (gh_number_p(ty), ty,  SCM_ARG5, "fltk-tile-position");

  widget = (Fl_Tile *) gh_cdr (widget_smob);

  widget->position(gh_scm2int(sx), gh_scm2int(sy), 
		   gh_scm2int(tx), gh_scm2int(ty));

  return SCM_UNSPECIFIED;
}


static SCM
fltk_tile_resize (SCM widget_smob, SCM x, SCM y, SCM w, SCM h)
{
  Fl_Tile *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_tile_tag),
              widget_smob, SCM_ARG1, "fltk-tile-resize");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-tile-resize");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-tile-resize");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG4, "fltk-tile-resize");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG5, "fltk-tile-resize");

  widget = (Fl_Tile *) gh_cdr (widget_smob);

  widget->resize(gh_scm2int(x), gh_scm2int(y),
    gh_scm2int(w), gh_scm2int(h));

  return SCM_UNSPECIFIED;
}


//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Tile *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Tile) + sizeof (Fl_Tile));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Tile *widget = (Fl_Tile *) gh_cdr (widget_smob);

  scm_puts ("#<widget Fl_Tile ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);

  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-tile scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_tile_init ()
{
  fltk_tile_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_group_subclass("fltk-tile", fltk_tile_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-tile-new",
		       fltk_tile_new);
  gh_new_procedure5_0 ("fltk-tile-position",
		       fltk_tile_position);
  gh_new_procedure5_0 ("fltk-tile-resize",
		       fltk_tile_resize);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_tile_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_tile_tag);
  fltk_button_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-tile-new"));
  sprintf (t, "%s-position", s);
  gh_define (t, scm_symbol_value0("fltk-tile-position"));
  sprintf (t, "%s-resize", s);
  gh_define (t, scm_symbol_value0("fltk-tile-resize"));
}
