// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Valuator.H>
#include "guile_fltk.h"

long fltk_valuator_tag;

//***************************************************************************
//   
//  Fl_Valuator encapsulation method(s):
//
//***************************************************************************

static SCM
fltk_valuator_get_value(SCM widget_smob)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-get-value");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  return gh_double2scm(widget->value());
}

static SCM
fltk_valuator_set_value(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-set-value");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-set-value");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->value(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}


static SCM
fltk_valuator_get_minimum(SCM widget_smob)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-get-minimum");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  return gh_double2scm(widget->minimum());
}

static SCM
fltk_valuator_set_minimum(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-set-minimum");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-set-minimum");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->minimum(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}


static SCM
fltk_valuator_get_maximum(SCM widget_smob)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-get-maximum");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  return gh_double2scm(widget->maximum());
}


static SCM
fltk_valuator_set_maximum(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-set-maximum");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-set-maximum");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  widget->maximum(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}


static SCM
fltk_valuator_range(SCM widget_smob, SCM a, SCM b)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-get-maximum");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-set-maximum");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG3, "fltk-valuator-set-maximum");

  double tmin = gh_scm2double(a);
  double tmax = gh_scm2double(b);

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->range(tmin, tmax);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_valuator_get_step(SCM widget_smob)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-get-step");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  return gh_double2scm(widget->step());
}

static SCM
fltk_valuator_set_step(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-set-step");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-set-step");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->step(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_valuator_format(SCM widget_smob)
{
  Fl_Valuator *widget;
  char buffer[80];

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-format");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->format(buffer);

  return scm_makfrom0str(buffer);
}

static SCM
fltk_valuator_round(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-round");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-round");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->round(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_valuator_clamp(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-clamp");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-round");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  widget->clamp(gh_scm2double(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_valuator_increment(SCM widget_smob, SCM a, SCM b)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-increment");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-increment");
  SCM_ASSERT (gh_number_p(b), b,  SCM_ARG2, "fltk-valuator-increment");

  double aa = gh_scm2double(a);
  int tn = gh_scm2int(b);
  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  widget->increment(aa, tn);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_valuator_get_changed(SCM widget_smob)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-get-changed");

  widget = (Fl_Valuator *) gh_cdr (widget_smob);
  if (widget->changed())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_valuator_set_changed(SCM widget_smob, SCM a)
{
  Fl_Valuator *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_valuator_tag),
              widget_smob, SCM_ARG1, "fltk-valuator-set_changed");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-valuator-increment");

  int tn = gh_scm2int(a);
  widget = (Fl_Valuator *) gh_cdr (widget_smob);

  if (tn)
    widget->set_changed();
  else
    widget->clear_changed();

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Valuator *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Valuator) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Valuator *widget = (Fl_Valuator *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Valuator ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-valuator scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_valuator_init ()
{
  fltk_valuator_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_widget_subclass("fltk-valuator", fltk_valuator_tag);

  // add new methods
  gh_new_procedure1_0 ("fltk-valuator-get-value", 
		       fltk_valuator_get_value);
  gh_new_procedure2_0 ("fltk-valuator-set-value", 
		       fltk_valuator_set_value);
  gh_new_procedure1_0 ("fltk-valuator-get-minimum", 
		       fltk_valuator_get_minimum);
  gh_new_procedure2_0 ("fltk-valuator-set-minimum", 
		       fltk_valuator_set_minimum);
  gh_new_procedure1_0 ("fltk-valuator-get-maximum", 
		       fltk_valuator_get_maximum);
  gh_new_procedure2_0 ("fltk-valuator-set-maximum", 
		       fltk_valuator_set_maximum);
  gh_new_procedure3_0 ("fltk-valuator-range", 
		       fltk_valuator_range);
  gh_new_procedure1_0 ("fltk-valuator-get-step", 
		       fltk_valuator_get_step);
  gh_new_procedure2_0 ("fltk-valuator-set-step", 
		       fltk_valuator_set_step);
  gh_new_procedure1_0 ("fltk-valuator-format", 
		       fltk_valuator_format);
  gh_new_procedure2_0 ("fltk-valuator-round", 
		       fltk_valuator_round);
  gh_new_procedure2_0 ("fltk-valuator-clamp", 
		       fltk_valuator_clamp);
  gh_new_procedure3_0 ("fltk-valuator-increment", 
		       fltk_valuator_increment);
  gh_new_procedure1_0 ("fltk-valuator-get-changed", 
		       fltk_valuator_get_changed);
  gh_new_procedure2_0 ("fltk-valuator-set-changed", 
		       fltk_valuator_set_changed);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_valuator_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_valuator_tag);
  fltk_widget_subclass(s, tag);

  sprintf (t, "%s-get-value", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-get-value"));
  sprintf (t, "%s-set_value", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-set-value"));
  sprintf (t, "%s-get-minimum", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-get-minimum"));
  sprintf (t, "%s-set-minimum", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-set-minimum"));
  sprintf (t, "%s-get-maximum", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-get-maximum"));
  sprintf (t, "%s-set-maximum", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-set-maximum"));
  sprintf (t, "%s-set-range", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-set-range"));
  sprintf (t, "%s-get-step", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-get-step"));
  sprintf (t, "%s-set_step", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-set-step"));
  sprintf (t, "%s-format", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-format"));
  sprintf (t, "%s-round", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-round"));
  sprintf (t, "%s-clamp", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-clamp"));
  sprintf (t, "%s-increment", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-increment"));
  sprintf (t, "%s-get-changed", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-get-changed"));
  sprintf (t, "%s-set-changed", s);
  gh_define (t, scm_symbol_value0("fltk-valuator-set-changed"));

}
