// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Window.H>
#include "guile_fltk.h"

long fltk_widget_tag;

//***************************************************************************
//   
//  Fl_Widget encapsulation method(s):
//
//***************************************************************************

static SCM
fltk_widget_x (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-x");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm(widget->x());
}

static SCM
fltk_widget_y (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-y");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm(widget->y());
}

static SCM
fltk_widget_w (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-w");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm(widget->w());
}

static SCM
fltk_widget_h (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-h");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm(widget->h());
}

static SCM
fltk_widget_resize (SCM widget_smob, SCM x, SCM y, SCM w, SCM h)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-resize");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-widget-resize");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-widget-resize");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG4, "fltk-widget-resize");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG5, "fltk-widget-resize");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->resize(gh_scm2int(x), gh_scm2int(y),
		 gh_scm2int(w), gh_scm2int(h));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_position (SCM widget_smob, SCM x, SCM y)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-position");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-widget-position");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-widget-position");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->position(gh_scm2int(x), gh_scm2int(y));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_size (SCM widget_smob, SCM w, SCM h)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-size");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG2, "fltk-widget-size");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG3, "fltk-widget-size");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->size(gh_scm2int(w), gh_scm2int(h));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_window (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-window");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gfltk_lookup((Fl_Widget *) widget->window());
}

static SCM
fltk_widget_set_box (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-box");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-widget-set-box");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->box((Fl_Boxtype)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_box (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-box");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->box());
}

static SCM
fltk_widget_set_type (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-type");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-widget-set-type");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->type((uchar)gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_type (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-type");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->type());
}

static SCM
fltk_widget_set_color (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-color");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-widget-set-color");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->color(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_color (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-color");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->color());
}

static SCM
fltk_widget_set_selection_color (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-selection-color");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-widget-set-selection-color");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->selection_color(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_selection_color (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-selection-color");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->selection_color());
}

static SCM
fltk_widget_set_label (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-label");
  SCM_ASSERT (gh_string_p(a), a, SCM_ARG2, "fltk-widget-new");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  if (widget->label() != NULL)
    free((char *)widget->label());

  widget->label(gh_scm2newstr(a, NULL));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_label (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-label");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_str02scm((char *)widget->label());
}

static SCM
fltk_widget_set_labeltype (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-labeltype");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-widget-set-labeltype");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->labeltype((Fl_Labeltype)gh_scm2long(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_labeltype (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-labeltype");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->labeltype());
}

static SCM
fltk_widget_set_align (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-align");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-widget-set-align");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->align(gh_scm2long(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_align (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-align");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->align());
}

static SCM
fltk_widget_set_labelcolor (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-labelcolor");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-widget-set-labelcolor");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->labelcolor(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_labelcolor (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-labelcolor");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->labelcolor());
}

static SCM
fltk_widget_set_labelfont (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-labelfont");
  SCM_ASSERT (gh_number_p(a), a, SCM_ARG2, "fltk-widget-set-labelfont");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->labelfont(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_labelfont (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-labelfont");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->labelfont());
}

static SCM
fltk_widget_set_labelsize (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-labelsize");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-widget-set-labelsize");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->labelsize(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_labelsize (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-labelsize");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->labelsize());
}

static SCM
fltk_widget_set_callback (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-callback");
  SCM_ASSERT (SCM_CLOSUREP (a), a,  SCM_ARG2, "fltk-widget-set-callback");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  gfltk_set_callback(widget, a);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_callback (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-callback");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gfltk_get_callback(widget);
}

static SCM
fltk_widget_set_user_data (SCM widget_smob, SCM sb)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-user-data");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->user_data((void *)sb);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_user_data (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-user-data");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return ((SCM)widget->user_data());
}

static SCM
fltk_widget_set_argument (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-argument");
  SCM_ASSERT (1, a, SCM_ARG2, "fltk-widget-set-argument");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->argument((long)a);

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_argument (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-argument");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return ((SCM)widget->argument());
}

static SCM
fltk_widget_set_changed (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-changed");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->set_changed();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_changed (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-changed");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  if (widget->changed())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_widget_clear_changed (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-clear-changed");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->clear_changed();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_set_when (SCM widget_smob, SCM a)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-set-when");
  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-widget-set-when");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->when(gh_scm2int(a));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_get_when (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-get-when");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->when());
}

static SCM
fltk_widget_visible (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-visible");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((long)widget->visible());
}

static SCM
fltk_widget_show (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-show");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->show();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_hide (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-hide");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->hide();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_active (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-active");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  if (widget->active())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_widget_activate (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-activate");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->activate();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_deactivate (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-deactivate");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->deactivate();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_redraw (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-redraw");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->redraw();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_widget_damage (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-damage");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gh_long2scm((int)widget->damage());
}

static SCM
fltk_widget_parent (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-damage");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  return gfltk_lookup((Fl_Widget *) widget->parent());
}

static SCM
fltk_widget_inside (SCM widget_smob, SCM widget_smob2)
{
  Fl_Widget *widget;
  Fl_Widget *widget2;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-inside");

  widget = (Fl_Widget *) gh_cdr(widget_smob);
  widget2 = (Fl_Widget *) gh_cdr(widget_smob2);

  if (widget->inside(widget2))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_widget_contains (SCM widget_smob, SCM widget_smob_2)
{
  Fl_Widget *widget;
  Fl_Widget *widget_2;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-contains");
  SCM_ASSERT (gfltk_type_p(widget_smob_2, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-contains");

  widget = (Fl_Widget *) gh_cdr(widget_smob);
  widget_2 = (Fl_Widget *) gh_cdr(widget_smob_2);

  if (widget->contains(widget_2))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_widget_take_focus (SCM widget_smob)
{
  Fl_Widget *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_widget_tag),
              widget_smob, SCM_ARG1, "fltk-widget-take-focus");

  widget = (Fl_Widget *) gh_cdr(widget_smob);

  widget->take_focus();

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Widget *) gh_cdr(widget_smob);
  gfltk_remove(widget_smob);
  return (sizeof (Gfltk) + sizeof (Fl_Widget));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Widget *widget = (Fl_Widget *) gh_cdr(widget_smob);
  scm_puts ("#<widget Fl_Widget ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-widget scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_widget_init ()
{
  fltk_widget_tag = scm_newsmob (&widget_smobfuns);

  // add new methods
  gh_new_procedure1_0 ("fltk-widget-x",
		       fltk_widget_x);
  gh_new_procedure1_0 ("fltk-widget-y",
		       fltk_widget_y);
  gh_new_procedure1_0 ("fltk-widget-w",
		       fltk_widget_w);
  gh_new_procedure1_0 ("fltk-widget-h",
		       fltk_widget_h);
  gh_new_procedure5_0 ("fltk-widget-resize",
		       fltk_widget_resize);
  gh_new_procedure3_0 ("fltk-widget-position",
		       fltk_widget_position);
  gh_new_procedure3_0 ("fltk-widget-size",
		       fltk_widget_size);
  gh_new_procedure1_0 ("fltk-widget-window",
		       fltk_widget_window);
  gh_new_procedure2_0 ("fltk-widget-set-box",
		       fltk_widget_set_box);
  gh_new_procedure1_0 ("fltk-widget-get-box",
		       fltk_widget_get_box);
  gh_new_procedure2_0 ("fltk-widget-set-type",
		       fltk_widget_set_type);
  gh_new_procedure1_0 ("fltk-widget-get-type",
		       fltk_widget_get_type);
  gh_new_procedure2_0 ("fltk-widget-set-color",
		       fltk_widget_set_color);
  gh_new_procedure1_0 ("fltk-widget-get-color",
		       fltk_widget_get_color);
  gh_new_procedure2_0 ("fltk-widget-set-selection-color",
		       fltk_widget_set_selection_color);
  gh_new_procedure1_0 ("fltk-widget-get-selection-color",
		       fltk_widget_get_selection_color);
  gh_new_procedure2_0 ("fltk-widget-set-label",
		       fltk_widget_set_label);
  gh_new_procedure1_0 ("fltk-widget-get-label",
		       fltk_widget_get_label);
  gh_new_procedure2_0 ("fltk-widget-set-labeltype",
		       fltk_widget_set_labeltype);
  gh_new_procedure1_0 ("fltk-widget-get-labeltype",
		       fltk_widget_get_labeltype);
  gh_new_procedure2_0 ("fltk-widget-set-align",
		       fltk_widget_set_align);
  gh_new_procedure1_0 ("fltk-widget-get-align",
		       fltk_widget_get_align);
  gh_new_procedure2_0 ("fltk-widget-set-labelcolor",
		       fltk_widget_set_labelcolor);
  gh_new_procedure1_0 ("fltk-widget-get-labelcolor",
		       fltk_widget_get_labelcolor);
  gh_new_procedure2_0 ("fltk-widget-set-labelfont",
		       fltk_widget_set_labelfont);
  gh_new_procedure1_0 ("fltk-widget-get-labelfont",
		       fltk_widget_get_labelfont);
  gh_new_procedure2_0 ("fltk-widget-set-labelsize",
		       fltk_widget_set_labelsize);
  gh_new_procedure1_0 ("fltk-widget-get-labelsize",
		       fltk_widget_get_labelsize);
  gh_new_procedure2_0 ("fltk-widget-set-callback",
		       fltk_widget_set_callback);
  gh_new_procedure1_0 ("fltk-widget-get-callback",
		       fltk_widget_get_callback);
  gh_new_procedure2_0 ("fltk-widget-set-user-data",
		       fltk_widget_set_user_data);
  gh_new_procedure1_0 ("fltk-widget-get-user-data",
		       fltk_widget_get_user_data);
  gh_new_procedure2_0 ("fltk-widget-set-argument",
		       fltk_widget_set_argument);
  gh_new_procedure1_0 ("fltk-widget-get-argument",
		       fltk_widget_get_argument);
  gh_new_procedure1_0 ("fltk-widget-set-changed",
		       fltk_widget_set_changed);
  gh_new_procedure1_0 ("fltk-widget-get-changed",
		       fltk_widget_get_changed);
  gh_new_procedure1_0 ("fltk-widget-clear-changed",
		       fltk_widget_clear_changed);
  gh_new_procedure2_0 ("fltk-widget-set-when",
		       fltk_widget_set_when);
  gh_new_procedure1_0 ("fltk-widget-get-when",
		       fltk_widget_get_when);
  gh_new_procedure1_0 ("fltk-widget-visible",
		       fltk_widget_visible);
  gh_new_procedure1_0 ("fltk-widget-show",
		       fltk_widget_show);
  gh_new_procedure1_0 ("fltk-widget-hide",
		       fltk_widget_hide);
  gh_new_procedure1_0 ("fltk-widget-active",
		       fltk_widget_active);
  gh_new_procedure1_0 ("fltk-widget-activate",
		       fltk_widget_activate);
  gh_new_procedure1_0 ("fltk-widget-deactivate",
		       fltk_widget_deactivate);
  gh_new_procedure1_0 ("fltk-widget-redraw",
		       fltk_widget_redraw);
  gh_new_procedure1_0 ("fltk-widget-damage",
		       fltk_widget_damage);
  gh_new_procedure1_0 ("fltk-widget-parent",
		       fltk_widget_parent);
  gh_new_procedure2_0 ("fltk-widget-inside",
		       fltk_widget_inside);
  gh_new_procedure2_0 ("fltk-widget-contains",
		       fltk_widget_contains);
  gh_new_procedure1_0 ("fltk-widget-take-focus",
		       fltk_widget_take_focus);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_widget_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_widget_tag);

  sprintf (t, "%s-x", s);
  gh_define (t, scm_symbol_value0("fltk-widget-x"));
  sprintf (t, "%s-y", s);
  gh_define (t, scm_symbol_value0("fltk-widget-y"));
  sprintf (t, "%s-w", s);
  gh_define (t, scm_symbol_value0("fltk-widget-w"));
  sprintf (t, "%s-h", s);
  gh_define (t, scm_symbol_value0("fltk-widget-h"));
  sprintf (t, "%s-resize", s);
  gh_define (t, scm_symbol_value0("fltk-widget-resize"));
  sprintf (t, "%s-position", s);
  gh_define (t, scm_symbol_value0("fltk-widget-position"));
  sprintf (t, "%s-size", s);
  gh_define (t, scm_symbol_value0("fltk-widget-size"));
  sprintf (t, "%s-window", s);
  gh_define (t, scm_symbol_value0("fltk-widget-window"));
  sprintf (t, "%s-set-box", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-box"));
  sprintf (t, "%s-get-box", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-box"));
  sprintf (t, "%s-set-type", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-type"));
  sprintf (t, "%s-get-type", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-type"));
  sprintf (t, "%s-set-color", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-color"));
  sprintf (t, "%s-get-color", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-color"));
  sprintf (t, "%s-set-selection-color", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-selection-color"));
  sprintf (t, "%s-get-selection-color", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-selection-color"));
  sprintf (t, "%s-set-label", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-label"));
  sprintf (t, "%s-get-label", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-label"));
  sprintf (t, "%s-set-labeltype", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-labeltype"));
  sprintf (t, "%s-get-labeltype", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-labeltype"));
  sprintf (t, "%s-set-align", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-align"));
  sprintf (t, "%s-get-align", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-align"));
  sprintf (t, "%s-set-labelcolor", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-labelcolor"));
  sprintf (t, "%s-get-labelcolor", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-labelcolor"));
  sprintf (t, "%s-set-labelfont", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-labelfont"));
  sprintf (t, "%s-get-labelfont", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-labelfont"));
  sprintf (t, "%s-set-labelsize", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-labelsize"));
  sprintf (t, "%s-get-labelsize", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-labelsize"));
  sprintf (t, "%s-set-callback", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-callback"));
  sprintf (t, "%s-get-callback", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-callback"));
  sprintf (t, "%s-set-user-data", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-user-data"));
  sprintf (t, "%s-get-user-data", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-user-data"));
  sprintf (t, "%s-set-argument", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-argument"));
  sprintf (t, "%s-get-argument", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-argument"));
  sprintf (t, "%s-set-changed", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-changed"));
  sprintf (t, "%s-get-changed", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-changed"));
  sprintf (t, "%s-clear-changed", s);
  gh_define (t, scm_symbol_value0("fltk-widget-clear-changed"));
  sprintf (t, "%s-set-when", s);
  gh_define (t, scm_symbol_value0("fltk-widget-set-when"));
  sprintf (t, "%s-get-when", s);
  gh_define (t, scm_symbol_value0("fltk-widget-get-when"));
  sprintf (t, "%s-visible", s);
  gh_define (t, scm_symbol_value0("fltk-widget-visible"));
  sprintf (t, "%s-show", s);
  gh_define (t, scm_symbol_value0("fltk-widget-show"));
  sprintf (t, "%s-hide", s);
  gh_define (t, scm_symbol_value0("fltk-widget-hide"));
  sprintf (t, "%s-active", s);
  gh_define (t, scm_symbol_value0("fltk-widget-active"));
  sprintf (t, "%s-activate", s);
  gh_define (t, scm_symbol_value0("fltk-widget-activate"));
  sprintf (t, "%s-deactivate", s);
  gh_define (t, scm_symbol_value0("fltk-widget-deactivate"));
  sprintf (t, "%s-redraw", s);
  gh_define (t, scm_symbol_value0("fltk-widget-redraw"));
  sprintf (t, "%s-damage", s);
  gh_define (t, scm_symbol_value0("fltk-widget-damage"));
  sprintf (t, "%s-parent", s);
  gh_define (t, scm_symbol_value0("fltk-widget-parent"));
  sprintf (t, "%s-inside", s);
  gh_define (t, scm_symbol_value0("fltk-widget-inside"));
  sprintf (t, "%s-contains", s);
  gh_define (t, scm_symbol_value0("fltk-widget-contains"));
  sprintf (t, "%s-take-focus", s);
  gh_define (t, scm_symbol_value0("fltk-widget-take-focus"));
}
