// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Window.H>
#include "guile_fltk.h"

long fltk_window_tag;

//***************************************************************************
//   
//  Fl_Check_Button encapsulation method(s):
//
//***************************************************************************

static SCM 
fltk_window_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Window * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-window-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-window-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-window-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-window-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-window-new");

  if ((x>=0) && (y>=0))
    widget = new Fl_Window(gh_scm2int(x), gh_scm2int(y), 
			   gh_scm2int(w), gh_scm2int(h),
			   gh_scm2newstr(label, NULL));
  else
    widget = new Fl_Window(gh_scm2int(w), gh_scm2int(h),
			   gh_scm2newstr(label, NULL));

  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_window_tag);
  gfltk_register(widget_smob);

  return widget_smob;
}

static SCM
fltk_window_show (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-show");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->show();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_hide (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-hide");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->hide();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_shown (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-shown");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  if (widget->shown())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_window_iconize (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-iconize");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->iconize();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_resize (SCM widget_smob, SCM x, SCM y, SCM w, SCM h)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-resize");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-window-resize");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-window-resize");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG4, "fltk-window-resize");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG5, "fltk-window-resize");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->resize(gh_scm2int(x), gh_scm2int(y), gh_scm2int(w), gh_scm2int(h));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_position (SCM widget_smob, SCM x, SCM y)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-position");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-window-position");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-window-position");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->position(gh_scm2int(x), gh_scm2int(y));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_size (SCM widget_smob, SCM w, SCM h)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-size");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG2, "fltk-window-size");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG3, "fltk-window-size");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->size(gh_scm2int(w), gh_scm2int(h));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_free_position (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-free-position");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->free_position();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_hotspot (SCM widget_smob, SCM x, SCM y, SCM off)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-hotspot");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-window-hotspot");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-window-hotspot");
  SCM_ASSERT (gh_number_p(off), off,  SCM_ARG4, "fltk-window-hotspot");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->hotspot(gh_scm2int(x), gh_scm2int(y), gh_scm2int(off));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_fullscreen (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-fullscreen");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->fullscreen();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_fullscreen_off (SCM widget_smob, SCM x, SCM y, SCM w, SCM h)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-fullscreen-off");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-window-fullscreen-off");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG3, "fltk-window-fullscreen-off");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG4, "fltk-window-fullscreen-off");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG5, "fltk-window-fullscreen-off");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->fullscreen_off(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_set_border (SCM widget_smob, SCM x)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-set-border");
  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG2, "fltk-window-set-border");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->border(SCM_NFALSEP(x));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_get_border (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-get-border");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  return gh_int2scm((int)widget->border());
}

static SCM
fltk_window_clear_border (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-clear-border");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->clear_border();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_set_modal (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-set-modal");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->set_modal();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_get_modal (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-get-modal");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  if (widget->modal())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_window_set_non_modal (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-set-non-modal");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  widget->set_non_modal();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_get_non_modal (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-get-non-modal");

  widget = (Fl_Window *) gh_cdr (widget_smob);
  if (widget->non_modal())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_window_set_label (SCM widget_smob, SCM label)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-set-label");
  SCM_ASSERT (SCM_NIMP (label) && SCM_STRINGP (label), label,
	      SCM_ARG2, "fltk-window-set-label");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  free((char *)widget->label());
  widget->label(gh_scm2newstr(label, NULL));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_get_label (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-get-label");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  return scm_makfrom0str (widget->label());
}

static SCM
fltk_window_set_iconlabel (SCM widget_smob, SCM label)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-set-iconlabel");
  SCM_ASSERT (SCM_NIMP (label) && SCM_STRINGP (label), label,
	      SCM_ARG2, "fltk-widow-set-iconlabel");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  if (widget->iconlabel() != NULL) 
    free((char *)widget->iconlabel());
  widget->iconlabel(gh_scm2newstr(label, NULL));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_get_iconlabel (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-get-iconlabel");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  return scm_makfrom0str (widget->iconlabel());
}

static SCM
fltk_window_set_xclass (SCM widget_smob, SCM label)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-set-xclass");
  SCM_ASSERT (SCM_NIMP (label) && SCM_STRINGP (label), label,
	      SCM_ARG2, "fltk-widow-set-icon-label");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->xclass(gh_scm2newstr(label, NULL));

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_get_xclass (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-get-xclass");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  return scm_makfrom0str (widget->xclass());
}

static SCM
fltk_window_make_current (SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-make-current");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  widget->make_current();

  return SCM_UNSPECIFIED;
}

static SCM
fltk_window_current (SCM widget_smob)//(SCM widget_smob)
{
  Fl_Window *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_window_tag),
              widget_smob, SCM_ARG1, "fltk-window-first-window");

  widget = (Fl_Window *) gh_cdr (widget_smob);

  return gfltk_lookup((Fl_Widget *) widget->current());
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Window *) gh_cdr (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Gfltk) + sizeof (Fl_Window));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Window *widget = (Fl_Window *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Window ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-window scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_window_init ()
{
  fltk_window_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_group_subclass("fltk-window", fltk_window_tag);

  // add new methods

  gh_new_procedure5_0 ("fltk-window-new*",
		       fltk_window_new);
  gh_eval_str("(define* (fltk-window-new label "
	      "#&key (x -1) (y -1) (w 100) (h 100)) "
	      "(fltk-window-new* x y w h label))");

  gh_new_procedure1_0 ("fltk-window-show",
		       fltk_window_show);
  gh_new_procedure1_0 ("fltk-window-hide",
		       fltk_window_hide);
  gh_new_procedure1_0 ("fltk-window-shown",
		       fltk_window_shown);
  gh_new_procedure1_0 ("fltk-window-iconize",
		       fltk_window_iconize);
  gh_new_procedure5_0 ("fltk-window-resize",
		       fltk_window_resize);
  gh_new_procedure1_0 ("fltk-window-free-position", 
		       fltk_window_free_position);
  gh_new_procedure4_0 ("fltk-window-hotspot",
		       fltk_window_hotspot);
  gh_new_procedure1_0 ("fltk-window-fullscreen",
		       fltk_window_fullscreen);
  gh_new_procedure5_0 ("fltk-window-fullscreen-off", 
		       fltk_window_fullscreen_off);
  gh_new_procedure2_0 ("fltk-window-set-border",
		       fltk_window_set_border);
  gh_new_procedure1_0 ("fltk-window-get-border",
		       fltk_window_get_border);
  gh_new_procedure1_0 ("fltk-window-clear-border",
		       fltk_window_clear_border);
  gh_new_procedure1_0 ("fltk-window-set-modal",
		       fltk_window_set_modal);
  gh_new_procedure1_0 ("fltk-window-get-modal",
		       fltk_window_get_modal);
  gh_new_procedure1_0 ("fltk-window-set-non-modal", 
		       fltk_window_set_non_modal);
  gh_new_procedure1_0 ("fltk-window-get-non-modal", 
		       fltk_window_get_non_modal);
  gh_new_procedure2_0 ("fltk-window-set-label",
		       fltk_window_set_label);
  gh_new_procedure1_0 ("fltk-window-get-label",
		       fltk_window_get_label);
  gh_new_procedure2_0 ("fltk-window-set-iconlabel", 
		       fltk_window_set_iconlabel);
  gh_new_procedure1_0 ("fltk-window-get-iconlabel", 
		       fltk_window_get_iconlabel);
  gh_new_procedure2_0 ("fltk-window-set-xclass",
		       fltk_window_set_xclass);
  gh_new_procedure1_0 ("fltk-window-get-xclass",
		       fltk_window_get_xclass);
  gh_new_procedure1_0 ("fltk-window-make-current",
		       fltk_window_make_current);
  gh_new_procedure1_0 ("fltk-window-current",
		       fltk_window_current);
  gh_new_procedure3_0 ("fltk-window-size",
		       fltk_window_size);
  gh_new_procedure3_0 ("fltk-window-position",
		       fltk_window_position);
}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_window_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_window_tag);
  fltk_group_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-window-new"));
  sprintf (t, "%s-show", s);
  gh_define (t, scm_symbol_value0("fltk-window-show"));
  sprintf (t, "%s-hide", s);
  gh_define (t, scm_symbol_value0("fltk-window-hide"));
  sprintf (t, "%s-shown", s);
  gh_define (t, scm_symbol_value0("fltk-window-shown"));
  sprintf (t, "%s-iconize", s);
  gh_define (t, scm_symbol_value0("fltk-window-iconize"));
  sprintf (t, "%s-resize", s);
  gh_define (t, scm_symbol_value0("fltk-window-resize"));
  sprintf (t, "%s-position", s);
  gh_define (t, scm_symbol_value0("fltk-window-free-position"));
  sprintf (t, "%s-hotspot", s);
  gh_define (t, scm_symbol_value0("fltk-window-hotspot"));
  sprintf (t, "%s", s);
  gh_define (t, scm_symbol_value0("-fltk-fullscreen window-fullscreen"));
  sprintf (t, "%s-fullscreen-off", s);
  gh_define (t, scm_symbol_value0("fltk-window-fullscreen-off"));
  sprintf (t, "%s-set-border", s);
  gh_define (t, scm_symbol_value0("fltk-window-set-border"));
  sprintf (t, "%s-get-border", s);
  gh_define (t, scm_symbol_value0("fltk-window-get-border"));
  sprintf (t, "%s-clear-border", s);
  gh_define (t, scm_symbol_value0("fltk-window-clear-border"));
  sprintf (t, "%s-set-modal", s);
  gh_define (t, scm_symbol_value0("fltk-window-set-modal"));
  sprintf (t, "%s-get-modal", s);
  gh_define (t, scm_symbol_value0("fltk-window-get-modal"));
  sprintf (t, "%s-set-non-modal", s);
  gh_define (t, scm_symbol_value0("fltk-window-set-non-modal"));
  sprintf (t, "%s-get-non-modal", s);
  gh_define (t, scm_symbol_value0("fltk-window-get-non-modal"));
  sprintf (t, "%s-set-label", s);
  gh_define (t, scm_symbol_value0("fltk-window-set-label"));
  sprintf (t, "%s-get-label", s);
  gh_define (t, scm_symbol_value0("fltk-window-get-label"));
  sprintf (t, "%s-set-iconlabel", s);
  gh_define (t, scm_symbol_value0("fltk-window-set-iconlabel"));
  sprintf (t, "%s-get-iconlabel", s);
  gh_define (t, scm_symbol_value0("fltk-window-get-iconlabel"));
  sprintf (t, "%s-set-xclass", s);
  gh_define (t, scm_symbol_value0("fltk-window-set-xclass"));
  sprintf (t, "%s-get-xclass", s);
  gh_define (t, scm_symbol_value0("fltk-window-get-xclass"));
  sprintf (t, "%s-make-current", s);
  gh_define (t, scm_symbol_value0("fltk-window-make-current"));
  sprintf (t, "%s-current", s);
  gh_define (t, scm_symbol_value0("fltk-window-current"));
  sprintf (t, "%s-size", s);
  gh_define (t, scm_symbol_value0("fltk-window-size"));
  sprintf (t, "%s-position", s);
  gh_define (t, scm_symbol_value0("fltk-window-position"));

}
